#include <cmath>
#include "basics.h"
#include "shape.h"
#include "c_types.h"

int step_checks;

/*
We have optimized check-collision, but apparently not enough.
*/

int shapeof(Object *x)
{
  shape *s;
  s=x->sh;
  if (s) return s->kind;
  return 0;
}

/*
Checks if the two parties in the net are going to have a collision
*/
c_notification check_collision(net_unit *n)
{
  Object *u1,*u2;
  int x1,x2,y1,y2, checks;
  float vx,vy;
  int t1,tnx1,tnx2,tny1,tny2,tbx1,tbx2,tby1,tby2;
  bool i1=false,i2;
   
  u1=n->p1; u2=n->p2; checks=n->kp.checks;

  if (checks & 16)
  {
//orep << "$$" << flush;
    c_notification cn1(n,4);
    cn1.time=0;
    cn1.arg=atan2(u1->vy-u2->vy,u1->vx-u2->vx);
    n->arg_eq(cn1.arg);
    if (fl2i(u1->vx)!=fl2i(u2->vx) || fl2i(u1->vy)!=fl2i(u2->vy))
      return cn1;
    return null_cn;
  }

  if (u1->togo==1 || u2->togo==1) return null_cn;
  if (u1->tlu>u2->tlu) u2->up_to_date(u1->tlu);
  else u1->up_to_date(u2->tlu);

  x1=u1->px; x2=u2->px;
  y1=u1->py; y2=u2->py;
  vx=u2->vx-u1->vx;
  vy=u2->vy-u1->vy;
  tnx1=u1->tnsx; tny1=u1->tnsy;
  tnx2=u2->tnsx; tny2=u2->tnsy;
  tbx1=u1->tbsx; tby1=u1->tbsy;
  tbx2=u2->tbsx; tby2=u2->tbsy;

  shape* s=combine_sh(u1->sh,u2->sh);

  t1=0;i2=0;

  if (checks&4 && !i1) // Quick filter, especially for lasers
  { 
    if (!s->kind) i2=1;
    else if (x2-x1>=s->max_x && u2->qx-u1->qx>=s->max_x) i2=1;
    else if (x2-x1<=s->min_x && u2->qx-u1->qx<=s->min_x) i2=1;
    else if (y2-y1>=s->max_y && u2->qy-u1->qy>=s->max_y) i2=1;
    else if (y2-y1<=s->min_y && u2->qy-u1->qy<=s->min_y) i2=1;
  }

  if (i2)
  {
    n->compromise=14;
    delete s;
    return null_cn;
  }

  i1=s->point_in(x2-x1,y2-y1);

  while (1)
  {
    step_checks++;

    t1=min4(tnx1,tnx2,tny1,tny2);
    if (t1>=0x10000)
    {
      delete s;
      return null_cn;
    }
    if (t1==tnx1)
    {
      if (u1->svx) x1--; else x1++;
      tnx1+=tbx1;
    }
    if (t1==tnx2)
    {
      if (u2->svx) x2--; else x2++;
      tnx2+=tbx2;
    }
    if (t1==tny1)
    {
      if (u1->svy) y1--; else y1++;
      tny1+=tby1;
    }
    if (t1==tny2)
    {
      if (u2->svy) y2--; else y2++;
      tny2+=tby2;
    }
    i2=s->point_in(x2-x1,y2-y1);

    if (i2 && !i1 && (checks&4)) // the following can be condensed
    {
      c_notification cn1(n,2);
      cn1.time=t1;
      cn1.arg=s->argument(x2-x1,y2-y1);
      n->arg_eq(cn1.arg);
      cn1.in_at_end=s->point_in(u2->qx-u1->qx,u2->qy-u1->qy);

      delete s;
      return cn1;
    }

    if (!i2 && i1 && (checks&8))
    {
      c_notification cn1(n,3);
      cn1.time=t1;
      cn1.arg=s->argument(x2-x1,y2-y1);
      n->arg_eq(cn1.arg);
      delete s;
      return cn1;
    }
    i1=i2;
  }
}

char touching(Object *p1,Object *p2)
{
  shape *s; char r;
  if (!p1 || !p2) return 0;
  s=combine_sh(p1->sh,p2->sh);
  if (!s)
    orep << "  Combined a null shape for touching  ";
  r=s->point_in(p2->px-p1->px,p2->py-p1->py);
  delete s;
  return r;
}
