#include "basics.h"
#include "c_solve.h"
#include "stage.h"

int blake_iterations;

/*            Adds x to the list of Objects and collects any eligible blakes from its net-units   */

int blatrix::find(Object *x)
{
  for (int j=0;j<(int)allObj.size();j++)
    if (x==allObj[j]) return j;
  orep << "Object not found in blatrix" << endl; exit(0);
}

void blatrix::collect(Object *x)
{
  net_unit *b2;

  if (x->collected) {orep << "Blatrix recollecting Object"; exit(0);}

  Objects.push_back(x);
  x->collected=1;

  for (b2=x->net_d;b2;b2=b2->d) //x=p1
  {
    if (!b2->b || b2->collected) {}
    else collect(b2);
  }
  for (b2=x->net_r;b2;b2=b2->r) //x=p2
  {
    if (!b2->b || b2->collected) {}
    else collect(b2);
  }
}

/*          Adds b to the list of blakes. If one of the Objects is not collected 
                   and will be moved by an impulse, it is collected.           */

void blatrix::collect(net_unit *b)
{
//  if (b->p1->kind==xk_Pushme && b->p2->kind==xk_Star)
//    if (((Pushme*)b->p1)->doubled) b->p1p=0; // cheat to stop stars pushing two

  if (b->collected) {orep << "Blatrix recollecting blake"; exit(0);}
  blakes.push_back(b);
  b->collected=1;
  if (!b->p1->collected && b->p1p!=0) collect(b->p1);
  if (!b->p2->collected && b->p1p!=1) collect(b->p2);
}

void blatrix::post_collect()
{
  for (int j=0;j<(int)blakes.size();j++)
  { net_unit* b=blakes[j];
    if (!b->p1->collected)
      allObj.push_back(b->p1);
    if (!b->p2->collected)
      allObj.push_back(b->p2);
  }
  for (int j=0;j<(int)Objects.size();j++)
    allObj.push_back(Objects[j]);    
}

void blatrix::list()
{
  int j;
  orep << "\nWe have the following Objects:\n";
  for (j=0;j<(int)allObj.size();j++)
  {
    Object* o=allObj[j];
    orep << j << ": k=" << o->kind << ", sh=" << (int)o->sh->kind << ", pos=" << o->px << "," << o->py << ", v=" << o->vx << "," << o->vy << "\n";
  }
  orep << "\nAnd we have the following blakes:\n";
  for (j=0;j<(int)blakes.size();j++)
  {
    net_unit* b=blakes[j];
    orep << find(b->p1) <<" & "<< find(b->p2) << ", p1p=" << b->p1p << ", arg=" << b->arg << "[" << b->cosarg << "\\" << b->sinarg << "], norm=" << b->norm_speed << ", shear=" << b->shear_speed << ", J=" << b->J << "\n";
  }
  orep << endl;
}

void blatrix::solve()
{
  float ns1;
  net_unit *b;

  int patience=1000; char made_change=-1;
// orep << ":" << flush;

//  for (int j=0;j<(int)blakes.size();j++)  We need something like that here i think
//    blakes[j]->vreset();
/*

  for (int j=0;j<(int)Objects.size();j++)        // This loop used to work out whether to squash any stars
    if (Objects[j]->kind==xk_Star)
    {
      Star* s1=(Star*)Objects[j];
      for (int k=0;k<(int)blakes.size();k++)
      {
        b=blakes[k];
        if (b->p2==s1)
        {
          if (b->arg>-3*PI/4 && b->arg<-PI/4)
            s1->floor_force=1;
          if (b->arg>PI/4 && b->arg<3*PI/4 && b->p1->vy>3)
            s1->above_force=1;
        }
      }
      if (s1->amisquashed())
      {
        made_change=0; s1->kill();
orep << "This complicated routine killed a star.\n";
      }
    }
*/
  while (patience && made_change)
  {
blake_iterations++;
    made_change=0;
    for(int j=0;j<(int)blakes.size();j++)
    {
      b=blakes[j];
      b->components();
      ns1=b->norm_speed;
      if (ns1>0.4)
      {
        if (ns1<1) b->J=0.2;
        else       b->J=ns1*0.2;
        made_change=1;
      }
      else
      {
        c_notification try_cn=check_collision(b);
        if (try_cn.b)
        {
          b->J=0.2;
          made_change=1;
        }
      }
    }

    if (patience<0) list();
    if (patience==-15) exit(0);

    for(int j=0;j<(int)blakes.size();j++)
    {
      b=blakes[j];
      b->impulse(1);
/*
      if (patience==950 && b->p1->kind==xk_Seismos)
      {
        b->p2->move(fl2i(2*b->cosarg),fl2i(2*b->sinarg)); // illegal, but if it works...
      }

      if (patience==850 && b->p1->kind==xk_Platform && b->p2->kind==xk_Star)
      {
//          b->p1->move(0,-1);
//          b->b=0;
//          blakes[j]=blakes[n_blakes];  What'll we do here?
//          n_blakes--;
      }
*/
    }      
    patience--;
    if (!patience) 
    {
      orep << "Out of patience. Resetting and replaying...\n";
      for (int j=0;j<(int)blakes.size();j++)
        blakes[j]->vreset();
      patience--;
    }
  }
  if (patience<0) orep << "SOLVED on 2nd attempt" << endl;
    
// orep << ";" << flush;
  for (int j=0;j<(int)Objects.size();j++)
    {
      Objects[j]->collected=0;
      owner->check_list.push_back(Objects[j]);
    }
    for (int j=0;j<(int)blakes.size();j++)  blakes[j]->collected=0;
  }

void blatrix::clean()
{
  blakes.clear();
  Objects.clear();
  allObj.clear();
}

void blatrix::solve(net_unit *b)
{
  clean();
  collect(b);
  post_collect();
  solve();
}

void blatrix::consider(Object *x)
{
  clean();
  collect(x);
  post_collect();
  solve();
}

void blatrix::go(c_notification cn,float n,float m,float ts)
{
  net_unit* b=cn.nu;
  if (cn.time==-3) n=1-n;
  b->new_blake(n,m,ts);
  if (Abs(ts)>0.2) b->set_limit(1.5); // Trick to limit star
 
  return solve(b);
}

blatrix WorkingMB;
