#ifndef _C_SOLVE_H_
#define _C_SOLVE_H_

#include <vector>
#include <set>
using namespace std;
#include "c_types.h"

class Stage;

/*
Blake refers to a colliding pair of objects.
Often we will have a system of one or more colliding pairs and in each case we want the velocities changed so no pair of objects overlap. Our means of changing velocities is by the impulse function in the net-unit class. We want to know which blakes to give impulses to and how big.
Mathematically this problem would be solved with matrix operations. We solve the matrix equation AB=C, where B[i] is the impulse to give blake i, C[i] is the normal speed of blake i which has to be set to zero and A[i,j] is the amount blake i will be pushed apart per unit of impulse given on blake j.
The mathematical solution couldn't be implemented due to rounding errors, but the blatrix (blake-matrix) class solves systems of blakes iteratively, executing small impulses of the blakes which require it until the system is solved.
*/


class blatrix {
  int find(Object*x);
  void collect(Object *x);
  void collect(net_unit *b);
  void post_collect();

  vector <net_unit*> blakes;
  vector <Object*> Objects;
  vector <Object*> allObj;

public:
  Stage *owner;

  void list();
  void solve();
  void clean();
  void solve(net_unit *b);
  void consider(Object *x);
  void go(c_notification cn,float n,float m,float ts);
};

//extern blatrix WorkingMB;

#endif 
