#ifndef _C_TYPES_H_
#define _C_TYPES_H_

struct c_reporter;
struct c_notification;

#include "net.h"
#include "msg.h"
#include "c_solve.h"


class Stage;
class blatrix;
message Sfx(int);

struct c_reporter
{
  Stage *owner;

  c_reporter& operator<<(Object *x);
  c_reporter& operator<<(message m);
  c_reporter& operator+ (Object *x);
  blatrix& Blatrix();
};

/*
Collision notification:
p1 and p2 are the parties involved
time is when during the time unit the collision occurs (0-0xffff)
b: 0 for no collision, 2 for an external collision, 3 for an internal collision, 4 or 5 for a chain constraint
in_at_end: ?? would they be overlapping at the end of the time unit?
nu : net unit which produced the report
arg: angle of collision
*/

struct c_notification
{ 
  Object *p1,*p2;
  int time;
  char b,in_at_end;
  net_unit *nu;
  float arg;

  c_notification();
  c_notification(net_unit *n,char b1);
  void deal_with(c_reporter &cr);
};

extern c_notification null_cn,valid_cn;

c_notification check_collision(net_unit *n);

char touching(Object *p1,Object *p2);



/*
The collision heap class is a binary heap (array) of collision notifications sorted by the time variable

some useful functions:
clear: empties the heap
add_cn: adds a collision notification to the heap
pullup: Use this to remove an element of the heap. If you understand binary heaps, you'llknow how this works.
remove_unwanted: removes all notifications with a certain Object from the heap
update_checks: The Object goes through all its net units, checking for collisions where appropriate (as in Physics_II). Any relevant notifications are added to the heap.
print_heap: Writes out the contents of the heap

The to element of the heap is a[1]. Use pullup(1) to remove it.
*/


#define c_heap_size 600

class collision_heap
{
public:
  c_notification a[c_heap_size];
  int heap_size;

  collision_heap();
  void add_cn(c_notification cn);
  void adjust_end();
  void pullup(int n);
  void remove_unwanted(int n,Object *p);
  void update_checks(Object *x);
  void print_h(int j);
  void print_heap();
};

#endif 
