#include <fstream>

#ifdef LINUX
#include <unistd.h>
#endif

#include "str.h"
#include "kbt.h"
#include "palettes.h"
#include "draw2.h"
#include "pics.h"
#include "go.h"
#include "puz.h"
#include "define.h"

int preffered_video_mode = GFX_AUTODETECT;

void set_pref_vidmode(int i)
{
	preffered_video_mode = i;
}

BITMAP *vidmode(int w,int h)
{
  set_gfx_mode(preffered_video_mode,w,h,0,0);
  BITMAP *r=create_bitmap(w,h);
  set_clip(r,0,0,w-1,h-1);
  return r;
}

BITMAP *to_screen_vidmode(int w,int h)
{
  BITMAP *r=create_bitmap(w,h);
  set_clip(r,0,0,w-1,h-1);
  return r;
}

void SimpleTimer()
{ Time++; }
END_OF_FUNCTION(SimpleTimer);
// This goes in Timing soon


void SolveProblems(Stage &S)
{
  for (Object* m=S.list;m;m=m->next)
  {
    if (m->kind==xk_Chain)
    {
      Chain* c=(Chain*)m;
      c->held=c->want_held;
    } 

    if (is_device(m))
    {
      Device* d=(Device*)m;
      d->on=d->want_on;
      if (d->kind==xk_Door2)
        ((Door2*)d)->reshape();
    }

    if (m->kind==xk_Star)
      ((Star*)m)->wake_up();
  }
}

void StageManager::set_screen_option(char o)
{
  if (to_screen)
    destroy_bitmap(to_screen);
/*  if (o==0)
    to_screen=NULL;
  if (o==1)
    to_screen=vidmode(320,200);*/
//  if (o==2)
  to_screen=to_screen_vidmode(640,480);
  screen_option=o;
  actual_brightness=-1;
  Timer=1;
}

  void StageManager::Load(string s)
  {
    Object *m;
    c1.load(s);
    for (m=c1.list;m;m=m->next)
      if (m->kind==xk_Player)
        p1=(Player*)m;

    if (recording)
    {
      char s1[30];
      delete oks;
      int n=0;
      do {
        n++;
#ifdef POSIX_DIR
        sprintf(s1,"fi5/%03d.rec",n);
#else
        sprintf(s1,"fi5\\%03d.rec",n);
#endif
      } while (exists(s1));

    oks=new ofstream(s1);
    *oks << s << endl;
    }
  }

  void StageManager::KeyS()
  {
    int k;
    string s1;

    rectfill(screen,30,80,289,119,173);
    text_mode(-1);
    rect(screen,30,80,289,119,1);
    textout_centre(screen,font,"Save in which position? (1-9)",160,84,1);
    for(k='1';k<='9';k++)
    {

#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif
	    
      s1+=L.prefix; s1+=k; s1+=".st1";
      textprintf(screen,font,45+25*(k-48),104,1+215*(!exists((char*)s1.c_str())),"%c",k);
    }
    k=KeyBoard.getkey();
    if (k<=KEY_9 && k>=KEY_1)
    {

#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif
	    
	s1+=L.prefix; s1+=(k-KEY_0+'0'); s1+=".st1";
      set_attribute(c1.header,"Created by: ","In-game save");
      c1.save(s1);
      if (recording)
        *oks << (int)(100+k-KEY_0) << " ";
    } 
    if (k==KEY_S || k==KEY_L)
      KeyBoard.enBuffer(k);
    k=0;
    Timer=1;
  }

  void StageManager::KeyL()
  {
    int k;
    string s1;

    rectfill(screen,30,80,289,119,172);
    rect(screen,30,80,289,119,1);
    text_mode(-1);
    textout_centre(screen,font,"Load from which position? (0-9)",160,84,1);
    for(k='0';k<='9';k++)
    {
      playback=false;
#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif
      s1+=L.prefix; s1+=k; s1+=".st1";
      if (exists((char*)s1.c_str()))
        textprintf(screen,font,45+25*(k-48),104,1,"%c",k);
    }
    k=KeyBoard.getkey();
    if (k>=KEY_0 && k<=KEY_9)
    {
      if (k==KEY_0) k='0';
      else k+='1'-KEY_1;
#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif
     
      s1+=L.prefix; s1+=k; s1+=".st1";
      if (exists((char*)s1.c_str()))
      {
        Load(s1);
      }
    }
    if (k==KEY_S || k==KEY_L)
      KeyBoard.enBuffer(k);
    k=0;
    Timer=1;
  }


    void StageManager::Run()
    {
      char k;

      if (!playback)
        ms.check_keys();

      while ((k=KeyBoard.deBuffer()))
      {
        if (!playback)  ms.check_key(k);     

        if (recording) ms.save_key(k,*oks);

        if (k==KEY_S)
          KeyS();
        if (k==KEY_L)
          KeyL();
        if (k==KEY_TILDE)
          playback=false;

        if (k==KEY_U) // PrintScreen if we can manage it.
          save_bmp("dump.bmp",to_screen,a_palette);

        if (k==KEY_PRTSCR)  { orep << "PrintScreen keypress recorded\n";  }

        if (k==KEY_Q)
          set_screen_option(0);
        if (k==KEY_Z)
          set_screen_option(1);
        if (k==KEY_A)
          set_screen_option(2);

        if (k>=KEY_1 && k<=KEY_9)
          set_timer_speed(160);

        if (k==KEY_ESC) // Exit level
          cog=0;

      }
      if (playback)
      {
        if (iks->eof()) 
          playback=false;
        else
          ms << *iks;
      }

      if (recording) ms.save_keys(*oks,&oms);
      oms=ms;

      p1->m=ms;

      if (cog==2 || cog==3)
      {
        brightness+=fade_speed;
        if (brightness>9999)
        {
          brightness=9999;
          cog=1;
        }
      }
      if (cog==4)
      {
        brightness-=fade_speed;
        if (brightness<0)
        {
          brightness=40; // we could use a separate timer
          cog=5;
        }
      }
      if (cog==5)
      {  brightness--; if (!brightness) cog=0; }

      if (cog%5)   // Important stuff happens here
      {
        M.Regular();
        c1.do_intelg_list();        step_checks=blake_iterations=0;
        c1.Physics_I();
        c1.Physics_II();
        c1.Physics_III();
      }

      if (p1->how_many_home==(int)p1->s.size())
      {
        if (cog && cog<4)
        {
          cog=4;
          if (!p1->how_many_dead) p1->cheerme=1;
        }
      }
      else if (cog==4) cog=2; // Gasp

      k=-1;

      while (!MQ.q.empty())
      {
        message &m=MQ.q.front();

        if (SoundOn && m.freq)
        {
          play_sample((SAMPLE*)D[u_Sound01-1+m.what].dat,255,128,m.freq*100,0);
          if (m.what==sfx_SuperButton)
          {
             SolveProblems(c1);
             for (int i=0;i<3;i++) 
               play_sample((SAMPLE*)D[u_Sound01-1+m.what].dat,255,128,m.freq*100,0);
             M.RequestSilence();
          }
        }
        if (m.what==sfx_Trumpet) M.RequestTrumpet();
        MQ.q.pop();
      }
    }

  void StageManager::Draw()
  {
    int cx=0,cy;
    pic_tree *pt;

    if (screen_option)
    {
      if (screen_option==1) {cx=160; cy=90; }
      else                  {cx=320; cy=220; }

      if (cog%5 && screen_option)
      {
        if (actual_brightness!=brightness)
        {
          shade_216(brightness);
          actual_brightness=brightness;
        }

        c1.do_draw_list(&(p1->vi));
        v=p1->vi;

        pt=new pic_tree;
        for (int j=0;j<(int)v.a.size();j++)
          ((view_element)v.a[j]).draw_thing(D,to_screen,cx,cy,pt);

        DrawTree(pt);

        DeleteTree(pt);
        text_mode(-1);

/* Framerate is not necessery for noemal user
        if (screen_option==2) textprintf(to_screen,font,170,466,43,"Frame rate: %d",FrameRate);
        else
        {
          textprintf(to_screen,font,2,121,216,"Fps: %d",FrameRate);
          textprintf(to_screen,font,1,121,1,"Fps: %d",FrameRate);
        }
*/	
      }
 
      else clear_to_color(to_screen,1);

      if (cog==2)
        textout_centre(to_screen,font,"Wait - not finished yet!",cx,15,217);

      if (cog>2)
        textout_centre(to_screen,font,(char*)L.name.c_str(),cx,15,217);

      if (cog>3)
        if (p1->how_many_dead)
          textprintf_centre(to_screen,font,cx,26,217,"%d star%c home, %d star%c lost", p1->how_many_home, 's'+(p1->how_many_home==1)*(' '-'s'), p1->how_many_dead, 's'+(p1->how_many_dead==1)*(' '-'s'));
        else textout_centre(to_screen,font,"Well done!  All stars home!",cx,26,217);
    }

//    if (screen_option==1)      blit(to_screen,screen,0,0,0,0,320,200);
    blit(to_screen,screen,0,0,0,0,SCRX,SCRY);

    if (!screen_option)
    {
//      rest(10);
//      if (Frames % (FrameRate+1)==FrameRate/2+1)
//        orep << " Frame rate=" << FrameRate << " in " << L.name << " with TimerSpeed=" << TimerSpeed << endl;
    }
  }

  StageManager::StageManager()
  {
    brightness=0;
    p1=NULL;
    oks=NULL; iks=NULL;
    to_screen=NULL;
  }

  void StageManager::Init()
  {
    actual_brightness=-1;

    string s1;
#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif

    
    s1+=L.prefix; s1+="0.st1";
    Load(s1);

    if (playback)
    {

#ifdef POSIX_DIR
      s1="fi5/";
#else
      s1="fi5\\";
#endif
      
      s1+=L.prefix; s1+=".rec";
      iks=new ifstream(s1.c_str());
      *iks >> s1;
    }

    if (!p1) { orep << "No player on stage"; exit(0); }

    s1=get_attribute(c1.header,"Music Number=");
    int music_num=(int)s1[0]-48;
    bool music_loop=(get_attribute(c1.header,"Music Loop=")[0]=='Y');

    M.Init(D,music_num,music_loop);
    if (SoundOn) M.Play();
    
    c1.subscribe(&MQ);

    cog=3;

    oms.clear();  // this might be done already
    ms.clear();

    keyboard_lowlevel_callback=onkeypres;
  }

StageManager::~StageManager()
{
  if (SoundOn) M.Stop();
  keyboard_lowlevel_callback=NULL;
  clear_keybuf();
}

void StageManager::Go()
{

  Timer=1;
  
//  LOCK_FUNCTION(SimpleTimer);
//  install_int_ex(SimpleTimer, BPS_TO_TIMER(200));

  while (cog)
  {
    while (Timer && cog)
    {
      Run();
      Timer--;
    }
    if (!cog) break;
//    if (0 && Time>=TimerSpeed)
//    {
//      set_timer_speed(TimerSpeed+1);
//      Frames=Time=0;
//    }

    if ((Frames%10)==1 || !jumpy) { 
	    Draw(); 
	    Frames++;
#ifdef LINUX
//	    yield_timeslice();
		usleep(1); //give up of process time
			   //better scheduling than yield
#endif
    }

    if (TimerSpeed==330) cog=0;
  }
}
