#include <set>
#include <iterator>
#include "high.h"
#include "pics.h"
#include "draw2.h"
#include "str.h"
#include "palettes.h"
#include "puz.h"
#include "o_1.h"
#include "view.h"
#include "stage.h"
#include "fio3.h"
#include "window.h"
#include "joda.h"

using namespace std;
/* This is the source file for JoDa, the puzzle editor.
Contents
Simple methods
Main message function
More simple methods
File menu
Edit menu
Insert menu
General stuff
Drawing
Constructor
*/

// Macros to make global functions which call their equivalent JoDaStuff function

#define MsgFnI(proc) int JoDaStuff::proc(int msg,DIALOG *d,int c)
#define MsgFnE(proc) MsgFnD(J_##proc) { return ((JoDaStuff*)d->dp3)->proc(msg,d,c); }
#define MenuFnI(proc) int JoDaStuff::proc(int flags)
#define MenuFnE(proc) int J_##proc() { return ((JoDaStuff*)active_menu->dp)->proc(active_menu->flags); }
#define VoidI(proc) void JoDaStuff::proc()
#define VoidE(proc) void J_##proc(void *j) { ((JoDaStuff*)j)->proc(); }
#define IntI(proc) int JoDaStuff::proc(int n)
#define IntE(proc) int J_##proc(int n,void *j) { return ((JoDaStuff*)j)->proc(n); }

#define seterator set<Object*>::iterator

// -------------------------- Simple methods ----------------------------------

  void JoDaStuff::arrow_scroll()
  {
    if (X-v.px<-230) v.px=X+230;
    if (X-v.px>229)  v.px=X-229;
    if (Y-v.py<-200) v.py=Y+200;
    if (Y-v.py>200)  v.py=Y-200;
  }

  Object* JoDaStuff::closest_Object()
  {
    Object *r,*m;
    int min_d,dd; 
  
    r=NULL;
    min_d=200;
    for (m=C.list;m;m=m->next)
    {
      dd=sq(X-m->px)+sq(Y-m->py);
      if (selectable(m) && dd<=min_d)
      {
        min_d=dd;
        r=m;
      }
    }
    for (m=C.list;m;m=m->next)
      if (m->kind==xk_Chain)
        if (((Chain*)m)->below==r && r)
          return m;
    return r;
  }

  bool JoDaStuff::selected(Object *x)
  {  return Selection.find(x)!=Selection.end(); }

  void JoDaStuff::unselect_all()
  {
    Selection.clear();
    x1=NULL;
    need_update=TRUE;
  }

  void JoDaStuff::select(Object *x)
  {
    Selection.insert(x);
    x1=x;
    need_update=TRUE;
  }

  void JoDaStuff::find_x1_selected()
  {
    if (Selection.empty()) x1=NULL;
    else x1=*Selection.begin();
  }

  void JoDaStuff::toggle_select(Object *x)
  {
    seterator n=Selection.find(x);
    if (selected(x))
      Selection.erase(n);
    else
      select(x);
    need_update=TRUE;
  }


/* Autoselect is used when using cut, copy and delete. Control boxes are bound to their device. Any teleport selects its partner.
*/
  void JoDaStuff::autoselect()
  {
    Object *m;
    Chain *a; Lever *l; Device *d;
    seterator i1,i2;

    for(i1=Selection.begin();i1!=Selection.end();i1=i2)
    {
      i2=i1; i2++; m=*i1;
      if (m->kind==xk_Ctrlbox)
        Selection.erase(i1); 
    }
    
    for(i1=Selection.begin();i1!=Selection.end();i1=i2)
    {
      i2=i1; i2++; m=*i1;
      if (m->kind==xk_Teleport)
        Selection.insert(((Teleport*)m)->partner);

      if (m->kind==xk_Ferris)
        for (int j=0;j<Ferris_n;j++)
          Selection.insert(((Ferris*)m)->a[j]);

      if (m->kind==xk_Chain)
      {
        a=(Chain*)m;
        if (a->below) Selection.insert(a->below);
      }
      if (m->kind==xk_Lever)
      {
        l=(Lever*)m;
        Selection.insert(l->a1);
        Selection.insert(l->a2);
      }
      if (is_device(m))
      {
        d=(Device*)m;
        if (d->mover)
          Selection.insert(d->mover);
      }
    }
  }

  void JoDaStuff::move_selection(int mx,int my)
  {
    Object *m;
    seterator i1,i2;
    for(i1=Selection.begin();i1!=Selection.end();i1=i2)
    {
      i2=i1; i2++; m=*i1;
      m->move(mx,my);
      if (m->kind==xk_Chain)
      {
        Chain* c=(Chain*)m;
        c->below->px=c->px;
        c->below->py=c->py+c->length;
      }
      m->reshape(); // is this appropriate for levers?
    }
  }

// ------------------------------  Main message function --------------------------------

MsgFnI(d_main)
{
  int sphd,mx,my,mb;
  Object *m,*close;

  p->sc=NULL;

  if (msg==MSG_DRAW)
    draw_stuff();

  if (msg==MSG_GOTMOUSE) return D_WANTFOCUS;
  if (msg==MSG_LOSTFOCUS) return D_REDRAW;

  if (msg==MSG_CLICK)
  {
    X=gui_mouse_x()+v.px-250;
    Y=gui_mouse_y()+v.py-247;
    X0=X; Y0=Y;
    mb=mouse_b;

    draw_rectangle=FALSE;
    if (mb==1)
    {
      close=closest_Object();

      if (keyCTRL)                  // Click: Left button with Ctrl
      {
        if (close) toggle_select(close);
      }
      else                          // Click: Left button without Ctrl
      {
        unselect_all();
        if ((x1=close)) select(x1);
        else draw_rectangle=TRUE;
      }
    }
    else if (mb==2)
      if (!keyCTRL && Selection.size()==1)   // Click: Right button without Ctrl
        unselect_all();

    get_mouse_mickeys(&mx,&my);
    scare_mouse();
    while (mouse_b==mb)
    {
      get_mouse_mickeys(&mx,&my);
      if (mb==1)                          // Drag: Left button
      {
        X+=mx;    Y+=my;
        arrow_scroll();
        if (draw_rectangle)              // --Drawing rectangle
        {
          unselect_all();
          for(m=C.list;m;m=m->next)
            if ((m->px<X0)!=(m->px<X))
              if ((m->py<Y0)!=(m->py<Y))
                if (selectable(m))
                  select(m);
          find_x1_selected();
        }
        else move_selection(mx,my);      // --Dragging selection
      }
      else                                   // Drag: Right button
      {
        X+=mx;    Y+=my;
        v.px+=mx;
        v.py+=my;
      }
      broadcast_dialog_message(MSG_DRAW,0);
    }
    position_mouse(X-v.px+250,Y-v.py+247);
    unscare_mouse();

    return D_REDRAW;
  }

  if (msg==MSG_WANTFOCUS) return D_WANTFOCUS;
  if (msg==MSG_GOTFOCUS) return D_REDRAW;

  if (msg==MSG_XCHAR)
  {
    int ck;
    ck=c>>8;
    draw_rectangle=0;
    return D_O_K;
  }

  if (msg==MSG_CHAR)
  {
    int ck;
    ck=c>>8;
    draw_rectangle=0;

    if (ck==KEY_TAB || ck==KEY_ESC || keyALT) return D_O_K;

    if (Selection.size()==1)
    {
      if (ck==KEY_Q) x1->key_Q();
      if (ck==KEY_A) x1->key_A();
      if (ck==KEY_O) x1->key_O();
      if (ck==KEY_P) x1->key_P();
    }

    if (keySHIFT) sphd=1;
    else if (keyCTRL) sphd=25;
    else sphd=5;

    if (ck==KEY_LEFT) { X-=sphd; move_selection(-sphd,0); arrow_scroll(); }
    if (ck==KEY_RIGHT){ X+=sphd; move_selection( sphd,0); arrow_scroll(); }
    if (ck==KEY_UP)   { Y-=sphd; move_selection(0,-sphd); arrow_scroll(); }
    if (ck==KEY_DOWN) { Y+=sphd; move_selection(0, sphd); arrow_scroll(); }

    if (ck==KEY_V && keyCTRL) return e_paste();
    if (ck==KEY_S && keyCTRL) return f_save();
    if (ck==KEY_TAB||ck==KEY_ESC) return D_O_K;
    if (ck==KEY_Z && keyCTRL) return e_undo();
    if (ck==KEY_DEL) return e_delete();
    if (ck==KEY_X && keyCTRL) return e_cut();
    if (ck==KEY_C && keyCTRL) return e_copy();

    if (ck==KEY_ENTER)
    {
      close=closest_Object();
      if (keyCTRL)
      {
        if (close) toggle_select(close);
      }
      else if (!Selection.empty()) unselect_all();
      else if (close) toggle_select(close);
    }
    return D_REDRAW+D_USED_CHAR;
  }
  return D_O_K;
}

// ---------------------------------- More simple methods ---------------------------------------

void JoDaStuff::centre_if_req()
{
  int dx,dy;

  if (x1)
  {
    dx=x1->px-v.px;
    dy=x1->py-v.py;
    if (abs(dx)>230 && abs(dy)<189) return;
    X=x1->px;Y=x1->py;
    v.px=X;v.py=Y;
  }
  //else maybe put at p1's coords
}


int d_textbox2_proc(int msg,DIALOG *d,int c)
{
  if (msg==MSG_DCLICK) return D_CLOSE;
  return d_textbox_proc(msg,d,c);
}

MenuFnI(about) {
  Window W2(2);
  DIALOG &d=*(W2.next());
  d.proc=d_textbox2_proc;
  d.y=10;
  d.w=639;
  d.h=469;
  d.fg=1;
  d.bg=173;
  d.d1=d.d2=1;
  d.dp=D[u_Help1].dat;
  do_dialog(W2.a,0);
  return D_REDRAW;
}

MenuFnE(about)



/* ----------------------- File menu - see bottom for contents ------------------------- */

void JoDaStuff::load(char *fn)
{
  C.load(string(fn));

  p=NULL; bg=NULL;

  music_number=get_attribute(C.header,"Music Number=")[0]-'0';
  music_loop=!(get_attribute(C.header,"Music Loop=")=="No");

  for (Object* m=C.list;m;m=m->next)
  {
    if (m->kind==xk_Player)
    {
      p=(Player*)m;
      v.px=X=p->px;
      v.py=Y=p->py;
    }
    if (m->kind==xk_Backg)
      bg=(Backg*)m;
  }
  if (!p || !bg) orep << "Bad stage loaded from " << fn << endl;

  unselect_all();
  X0=Y0=0;
  inserting=draw_rectangle=false;

/*
      This doesn't do anything about a bad stage: ie no background or no player
*/
} 

void JoDaStuff::save(char *fn)
{
  if (music_loop) set_attribute(C.header,"Music Loop=","Yes");
  else set_attribute(C.header,"Music Loop=","No");
  string s1="0";
  s1[0]+=music_number;
  set_attribute(C.header,"Music Number=",s1);
  set_attribute(C.header,"Created by: ","JoDa");

  p->togo=0;
  p->current_Star=0;
  for(Object* m=C.list;m;m=m->next)
    if (m->kind==xk_Star)
      p->s.push_back((Star*)m);
  C.save(string(fn));
}

void JoDaStuff::Ado()
{ save("current.st1");
  CanUndo=true; }

MenuFnI(f_new) 
{ 
  fn[0]=0;
  music_number=0;
  music_loop=TRUE;
  C.delete_list();
  C.header="Puzzle Stars Stage";
  p=new Player;
  bg=new Backg;
  C.add_Object(bg);
  C.add_Object(p);
  v.px=X=X0=0;
  v.py=Y=Y0=0;
  inserting=draw_rectangle=false;
  unselect_all();
  Ado();
  return D_REDRAW;
}

MenuFnI(f_load)
{
  char fn1[80],fn2[80];

  fn1[0]=0; Ado();
  if(file_select("Load Stage:",fn1,"st1"))
  {
    replace_extension(fn2,fn1,"st1",80);
    if (exists(fn2))
    {
      load(fn2);
      strcpy(fn,fn2);
    }
  }
  return D_REDRAW;
}

MenuFnI(f_save) {
  if (fn[0])
    save(fn);
  return D_REDRAW;
}

MenuFnI(f_saveas)
{
  char fn1[80],fn2[80];

  fn1[0]=0;
  if(file_select("Save Stage As:",fn1,"st1"))
  {
    replace_extension(fn2,fn1,"st1",80);
    save(fn2);
    strcpy(fn,fn2);
  }
  return D_REDRAW;
}

MenuFnI(f_hed)
{
  C.header="Puzzle Stars Stage";
  return D_O_K;
}

MenuFnI(f_quit) {
//  sf_save("current.st1");
  return D_CLOSE;
}


MenuFnE(f_new)
MenuFnE(f_load)
MenuFnE(f_save)
MenuFnE(f_saveas)
MenuFnE(f_hed)
MenuFnE(f_quit)

MENU file_menu[]=
{
  { "&New",J_f_new,NULL,0,NULL},
  { "&Load",J_f_load,NULL,0,NULL},
  { "&Save\tCtrl-S",J_f_save,NULL,0,NULL},  // Condition: fn[0] (there's a filename)
  { "Save &As",J_f_saveas,NULL,0,NULL},
  { "Clean &Header",J_f_hed,NULL,0,NULL},
  { "E&xit\tEsc",J_f_quit,NULL,0,NULL},
  { NULL,NULL,NULL,0,NULL }
};

MENU help_menu[]=
{
  { "&Not much here",J_about,NULL,0,NULL},
  { NULL,NULL,NULL,0,NULL }
};


/* End of file menu
 -------------------------- Edit menu - see bottom for contents ---------------------------- */


#define ForSel for(i1=Selection.begin();i1!=Selection.end();i1=i2) { i2=i1; i2++; m=*i1;


MenuFnI(e_undo) {
  if (!CanUndo) return D_O_K;
  load("current.st1");
  CanUndo=false;
  return D_REDRAW;
}

MenuFnI(e_cut) {
  Object *m;
  seterator i1,i2;

  if (Selection.empty()) return D_O_K;
  Ado();
  autoselect();

  ForSel
    m->px-=X; m->py-=Y;
  }
  saver S("cb.pzs","Puzzle Stars Clipboard");
  S << (int)Selection.size();

  ForSel
    S << m;
  }
  S.complete();

  ForSel
    C.remove_Object(m);
    delete m;
  }
  unselect_all();
  return D_REDRAW;
}

MenuFnI(e_copy) {
  Object *m;
  seterator i1,i2;
  if (Selection.empty()) return D_O_K;
  Ado();
  SSpare=Selection;

  ForSel
    m->px-=X; m->py-=Y;
  }
  saver S("cb.pzs","Puzzle Stars Clipboard");
  S << (int)Selection.size();

  ForSel
    S << m;
  }
  S.complete();

  ForSel
    m->px+=X; m->py+=Y;
  }
  Selection=SSpare;
  return D_REDRAW;
}

MenuFnI(e_paste) {
  string s1;
  int j,n;
  vector <Object*> O;
  Object *m;

  Ado();
  unselect_all();
  loader L("cb.pzs",s1);
  L >> n;
  for (j=0;j<n;j++)
  {
    L >> m;
    O.push_back(m);
  }
  L.complete();
  for (j=0;j<n;j++)
  {
    m=O[j];
    C.add_Object(m);
    m->px+=X; m->py+=Y;
  }
  for (j=0;j<n;j++)
  {
    m=O[j];
    m->reshape();
//    if (selectable(m)) select(m);
  }
  return D_REDRAW;
}

MenuFnI(e_delete) {
  if (!x1) return D_O_K;

  Ado();
  Object *m; seterator i1,i2;

  autoselect();

  ForSel
    C.remove_Object(m);
    delete m;
  }
  unselect_all();
  return D_REDRAW;
}


MenuFnE(e_undo)
MenuFnE(e_cut)
MenuFnE(e_copy)
MenuFnE(e_paste)
MenuFnE(e_delete)


MENU edit_menu[]=
{                                          // Conditions:
  { "&Undo\tCtrl-Z", J_e_undo,NULL,0,NULL},  // CanUndo flag
  { "Cu&t\tCtrl-X",  J_e_cut,NULL,0,NULL},   // Something is selected
  { "&Copy\tCtrl-C", J_e_copy,NULL,0,NULL},  // Something is selected
  { "&Paste\tCtrl-V",J_e_paste,NULL,0,NULL}, // Something is in the clipboard & nothing is selected
  { "&Delete\tDel",  J_e_delete,NULL,0,NULL},// Something is selected
  { NULL,NULL,NULL,0,NULL }
};


/* end of edit menu
 ---------------------------- Insert menu - see bottom for contents ------------------------------ */


#define MeNewX(name,k) { name,J_i_newX,NULL,D_USER*k,NULL}
#define MeNewS(name,w) { name,J_i_newS,NULL,D_USER*w,NULL}
#define MeNewC(name,k) { name,J_i_newC,NULL,D_USER*k,NULL}
#define MeMenu(name,M) { name,NULL,M,0,NULL}
#define MenuEnd { NULL,NULL,NULL,0,NULL }


IntI(add_x)
{
  x1=new_x(n);
  x1->locate(X,Y);
  C.add_Object(x1);
  select(x1);
  return D_REDRAW;
}

IntI(add_setp)
{
  add_x(xk_Setpiece);
  ((Setpiece*)x1)->what=n;
  return D_REDRAW;
}

IntI(add_Lever)
{
  Platform *p1,*p2;
  
  p1=new Platform;
  p2=new Platform;
  C.add_Object(p1);
  C.add_Object(p2);
  add_x(xk_Lever);
  Lever* l=(Lever*)x1;
  l->a1=p1;
  l->a2=p2;
  return D_REDRAW;
}

IntI(add_Ferris)
{
  Platform *p;
  add_x(xk_Ferris);
  Ferris* f=(Ferris*)x1;
  for (int j=0;j<Ferris_n;j++)
  {
    p=new Platform;
    f->a[j]=p;
    C.add_Object(p);
  }
  f->reshape();
  x1=f;
  return D_REDRAW;
}

IntI(add_Teleport)
{
  Teleport *t1,*t2;
  add_x(xk_Teleport);
  t1=(Teleport*)x1;
  add_x(xk_Teleport);
  t2=(Teleport*)x1;
  t1->partner=t2;
  t2->partner=t1;
  t1->move(-50,0);
  return D_REDRAW;
}

IntI(add_Ctrlbox)
{
  add_x(xk_Ctrlbox);
  Ctrlbox *c=(Ctrlbox*)x1;
  if (n==xk_Ctrlbox)
  {
    c->super=1;
    return D_REDRAW;
  }
  add_x(n);
  c->move(-50,0);
  if (n==xk_Magnet) c->dev=c->mag=(Magnet*)x1;
  if (n==xk_Zapper) c->dev=c->zap=(Zapper*)x1;
  if (n==xk_Door2) c->dev=c->trap=(Door2*)x1;
  if (n==xk_Laser) c->dev=c->laz=(Laser*)x1; 
 return D_REDRAW;
}

VoidI(add_Chain)
{
  Object* x2=x1;
  add_x(xk_Chain);
  toggle_select(x2);
  Chain* c=(Chain*)x1;
  c->below=x2;
  c->move(0,-c->length);
}


MenuFnI(i_newX)
{ return add_x(flags/D_USER); }

MenuFnI(i_newS)
{ return add_setp(flags/D_USER); }

MenuFnI(i_newC)
{ return add_Ctrlbox(flags/D_USER); }

MenuFnE(i_newX)
MenuFnE(i_newS)
MenuFnE(i_newC)
MenuFnE(add_Lever)
MenuFnE(add_Ferris)
MenuFnE(add_Teleport)

VoidE(add_Chain)

MENU gettable_menu[]=
{
  MeNewX("&Key"     ,xk_Quay),
  MeNewX("&Cutter"   ,xk_Cutter),
  MeNewX("&Mallet"    ,xk_Mallet),
  MeNewX("&Trumpet"    ,xk_Trumpet),
  MeNewX("&Baseball bat",xk_Bat),
MenuEnd };

MENU setpiece_menu[]=
{
  MeNewS("&Left Curve" ,4),
  MeNewS("&Right Curve",5),
  MeNewS("&Decoration",6),
MenuEnd };

MENU device_menu[]=
{
  MeNewC("&Magnet",xk_Magnet),
  MeNewC("&Electric field",xk_Zapper),
  MeNewC("&Trapdoor",xk_Door2),
  MeNewC("&Laser",xk_Laser),
  MeNewC("&Super Button",xk_Ctrlbox),
MenuEnd };

MENU special_menu[]=
{
  {"&Ferris Wheel",J_add_Ferris,NULL,0,NULL},
  MeNewX("&Seismos" ,xk_Seismos),
  MeNewX("&Iceblock",xk_Iceblock),
MenuEnd };

MENU insert_menu[]=
{
  MeNewX("&Rectangle\tR",xk_Setp2),
  MeMenu("&Gettable\t>",gettable_menu),
  MeMenu("&Set Piece\t>",setpiece_menu),
  MeMenu("&Device\t>",device_menu),
  MeNewX("St&ar"   ,xk_Star),
  MeNewX("&Home"  ,xk_Home),
  MeNewX("D&oor"  ,xk_Door),
  MeNewX("&Mirror",xk_Mirror),
  {"&Teleport",J_add_Teleport,NULL,0,NULL},
  {"&Lever",J_add_Lever,NULL,0,NULL},
  MeNewX("Spi&kes",xk_Spikes),
  MeNewX("&Push-Me block",xk_Pushme),
  MeNewX("Hea&vyweight"  ,xk_Heavy),
  MeNewX("&Gong"  ,xk_Gong),
  MeMenu("Sp&ecial\t>",special_menu),
MenuEnd };


// ---------------------------------------- General stuff ---------------------------------

VoidI(set_start_position)
{  p->px=X;  p->py=Y; }

VoidI(insert_button)
{ inserting=true; need_update=TRUE; }

VoidI(not_inserting)
{ inserting=false; need_update=TRUE; }

VoidE(set_start_position)
VoidE(insert_button)
VoidE(not_inserting)

void set_flag(bool cond,MENU &m,int flag)
{
  if (cond) m.flags|=flag;
  else      m.flags&=~flag;
}


MENU top_menu[]=
{
  { "&File", NULL,file_menu,0,NULL},
  { "&Edit", NULL,edit_menu,0,NULL},
  { "&Insert",NULL,insert_menu,0,NULL},
  { "&Help", J_about,NULL,0,NULL},
  { NULL,NULL,NULL,0,NULL }
};

MsgFnE(d_main)

DIALOG mainD=  { J_d_main     , 0,0, 500, 479, 216, 216, 0, 0, 0, 0, NULL,    NULL, NULL };
DIALOG menuD=  { d_menu_proc, 0,0, 500, 12,   6,   6,   0, 0, 0, 0, top_menu,NULL, NULL };

MsgFnI(d_icon_add)
{
  if (msg==MSG_DRAW)
  {
    rectfill(screen,d->x,d->y,d->x+d->w-1,d->y+d->h-1,c3(135));
    if (d->flags & D_GOTFOCUS)
      rect(screen,d->x,d->y,d->x+d->w-1,d->y+d->h-1,c3(550));
    draw_sprite(screen,(BITMAP*)D[d->d2].dat,d->x+1,d->y+1);
    return D_O_K;
  }

  if (msg==MSG_WANTFOCUS) return D_WANTFOCUS;
  if (msg==MSG_GOTFOCUS)
  {
    strcpy(What_To_Insert,(char*)d->dp);
    return D_REDRAW;
  }
  if (msg==MSG_LOSTFOCUS)
  {
    What_To_Insert[0]=0;
    return D_REDRAW;
  }

  if (msg==MSG_CLICK||msg==MSG_KEY)
    return ((int(*)(int,void*)) d->dp2)(d->d1,this);

  return D_O_K;
}

MsgFnE(d_icon_add)

void JoDaStuff::AddIcon(int x,char *s,int p,int(*proc)(int,void*),int n)
{
  DIALOG *d=W.next();
  BITMAP *b=(BITMAP*)D[p].dat;
  d->proc=J_d_icon_add;
  d->x=x;
  d->y=W.y_level;
  d->h=b->h+2;
  d->w=b->w+2;
  d->dp2=(void*)proc;
  d->dp=(void*)s;
  d->d1=n;
  d->d2=p;
}

IntE(add_x)
IntE(add_Ctrlbox)
IntE(add_setp)
IntE(add_Lever)
IntE(add_Ferris)
IntE(add_Teleport)

#define AddIconX(x,name,pic,k) AddIcon(x,name,pic,J_add_x,k)
#define AddIconC(x,name,pic,k) AddIcon(x,name,pic,J_add_Ctrlbox,k)
#define AddIconS(x,name,pic,w) AddIcon(x,name,pic,J_add_setp,w)

/* ------------------------------ Drawing ---------------------------------------- */

void JoDaStuff::draw_stuff()
{
  pic_tree *pt;
  int cx,cy;
  Object *m;
  char S1[30];

  if (need_update)
  {
// We should delete spurious stringuds.

    W.clear();
    *(W.next())=mainD;
    *(W.next())=menuD;
    W.y_level=25;

    if (Selection.empty())
      if (inserting)
      {
        W.WrStr("Insert object");

        AddIconX(503,"Key",u_Key4,xk_Quay);
        W.Ln(25);

        AddIconC(503,"Trapdoor",u_Trap1,xk_Door2);
        AddIconC(547,"Electric Field",u_Zap0,xk_Zapper);
        AddIconC(581,"Laser",u_Laser1,xk_Laser);
        AddIconX(615,"Mirror",u_Mirror1a,xk_Mirror);
        W.Ln(25);

        AddIconC(505,"Magnet",u_Mag1,xk_Magnet);
        AddIcon(541,"Teleports",u_Telepic0,J_add_Teleport,0);
        AddIconX(567,"Spikes",u_Spike,xk_Spikes);
        AddIcon(585,"Lever",u_Lever2,J_add_Lever,0);
        W.Ln(35);

        AddIconX(505,"Star",u_Star3000,xk_Star);
        AddIconX(540,"Home",u_Home1,xk_Home);
        AddIconX(580,"Push-Me block",u_PushMe,xk_Pushme);
        AddIconX(615,"Heavyweight",u_Heavy,xk_Heavy);
        W.Ln(35);

        AddIconS(502,"Curved wall",u_Curve1,4);
        AddIconS(545,"Curved wall",u_Curve2,5);
        AddIconS(588,"Decoration",u_Decor,6);
        W.Ln(45);

        AddIconX(505,"Door",u_Door4,xk_Door);
        AddIconX(520,"Rectangle",u_REct,xk_Setp2);
        W.Ln(55);

        W.WrStr("Current Position");
        W.NumEdit(" X ",&X,s_int);
        W.NumEdit(" Y ",&Y,s_int);
        W.Ln();
        W.AddButton("Level Properties",J_not_inserting);
        W.Ln();
        W.WrStr(What_To_Insert);
        while (mouse_b) {}
      }

      else
      {
        W.WrStr("Level Properties");
        W.Ln();
        W.WrStr("Start Position");
        W.NumEdit(" X ",&p->px,s_int);
        W.NumEdit(" Y ",&p->py,s_int);
        W.AddButton("Set Position",J_set_start_position);
        W.Ln();
        W.WrStr("Background colour");
        W.NumEdit(" ",&bg->colour,s_colour);
        W.Ln();
        W.NumEdit("Music number: ",&music_number,s_int);
        W.BoolEdit("Music loop: ",&music_loop);
        W.Ln();
        W.WrStr("Current Position");
        W.NumEdit(" X ",&X,s_int);
        W.NumEdit(" Y ",&Y,s_int);
        W.Ln();
        W.AddButton("Insert Object",J_insert_button);
      }

    else if (Selection.size()==1)
      x1->edit(&W);

    else
    {
      W.WrStr("Selected:");
      sprintf(S1," %d objects",(int)Selection.size());
      W.WrStr(S1);
    }

    W.set_dp3((void*)this);

    need_update=FALSE;
  }
  if (Selection.size()==1) x1->reshape();

  v.clear();
  C.do_draw_list(&v);

  cx=250;cy=247;

  pt=new pic_tree;
  for (int k=0;k<(int)v.a.size();k++)
    ((view_element)v.a[k]).draw_thing(D,to_screen,cx,cy,pt);

  for (seterator i1=Selection.begin();i1!=Selection.end();i1++)
  {
    m=*i1;
    enTree(dc_sprite,14,pt,4,to_screen,D[u_Arrow2].dat,m->px+cx-v.px-10,m->py+cy-v.py);
  }
  if (Selection.empty()) enTree(dc_sprite,14,pt,5,to_screen,D[u_CrosHair].dat,X+cx-v.px,Y+cy-v.py,1);

  set_clip(to_screen,0,13,499,479);

  DrawTree(pt);
  DeleteTree(pt);

  if (draw_rectangle)
    rect(to_screen,X+cx-v.px,Y+cy-v.py,X0+cx-v.px,Y0+cy-v.py,1);

  set_clip(to_screen,0,0,639,479);
    

  vline(to_screen,500,0,479,1);
  rectfill(to_screen,501,0,639,479,c3(444));
  blit(to_screen,screen,0,14,0,14,640,468);
}

/* end of Drawing
 -------------------------------------------- Constructor ---------------------------- */

JoDaStuff::JoDaStuff(char *fn1,DATAFILE *D1):W(50)
{
  D=D1;

  vidmode(640,480);
  to_screen=create_bitmap(640,480);

  shade_216(9999);
  v.sx=250;v.sy=233;

  if (fn1) load(fn1); 
  else if (exists("current.st1")) load("current.st1");
  else f_new();

  CanUndo=0;
  set_mouse_sprite((BITMAP*)D[u_Mouse].dat);
  set_mouse_sprite_focus(9,0);
  show_mouse(screen); // why?
  *(W.next())=mainD; 
  *(W.next())=menuD;
  W.set_dp3((void*)this);
}

JoDaStuff::~JoDaStuff()
{
  destroy_bitmap(to_screen);
}

int JoDaMain(char *fn,DATAFILE *D)
{
  JoDaStuff J(fn,D);
  return do_dialog(J.W.a,0);
}
