
#include <cmath>
#include "basics.h"
#include "net.h"

kind_pair kinpair[max_xkinds*max_xkinds];

net_unit::net_unit(Object *o1,Object *o2,kind_pair kp1)
{
  p1=o1;
  p2=o2;
  kp=kp1;
  b=0;
  l=u=NULL;
  d=p1->net_d;
  r=p2->net_r;
  if (d) d->u=this;
  if (r) r->l=this;
  p1->net_d=this;
  p2->net_r=this;
}

void net_unit::components()
{
  shear_speed=(p2->vx-p1->vx)*sinarg-(p2->vy-p1->vy)*cosarg-target_shear;
  norm_speed=-(p2->vx-p1->vx)*cosarg-(p2->vy-p1->vy)*sinarg;
}

void net_unit::arg_eq(float n_arg)
{
  arg=n_arg;
  cosarg=cos(arg);
  sinarg=sin(arg);
}

void net_unit::new_blake(float n,float m,float ts)
{
  collected=compromise=0;
  b=1;
  p1p=n;
  mu=m;
  target_shear=ts;
  components();
  limited=0;
}

void net_unit::set_limit(float l)
{
  limited=1; limit=l;
}

void net_unit::vreset()
{
  p1->vreset(); p2->vreset();
  if (limited) limit=1.5;
  components();
}

void net_unit::impulse(char friction_on)
{
  float J1m,Jp1,Jmu,S1m,Sp1;
   if (J==0) return;
  if (kp.checks&8) J=-J;
  Jp1=J*p1p;   J1m=J*(1-p1p);
  p1->vx-=cosarg*Jp1;
  p1->vy-=sinarg*Jp1;
  p2->vx+=cosarg*J1m;
  p2->vy+=sinarg*J1m;

  if (friction_on)
  {
    Jmu=J*mu;
    if (limited && Jmu>limit) Jmu=limit;
    if (shear_speed<-Jmu) Jmu=-Jmu;
    else if (shear_speed<Jmu) Jmu=shear_speed;
    if (limited) limit-=Abs(Jmu);
    Sp1=p1p*Jmu; S1m=(1-p1p)*Jmu;
    p1->vx+=Sp1*sinarg;
    p1->vy-=Sp1*cosarg;
    p2->vx-=S1m*sinarg;
    p2->vy+=S1m*cosarg;
  }

  J=0;
  components();
  p1->calibrate();
  p2->calibrate();
}

void net_unit::delete_d()
{
  if (r) r->l=l;
  if (l) l->r=r;
  else p2->net_r=r;
  if (d) d->delete_d();
  delete this;
}

void net_unit::delete_r()
{
  if (u)  u->d=d;
  else p1->net_d=d;
  if (d) d->u=u;
  if (r) r->delete_r();
  delete this;
}

void Object::de_net()
{
  if (net_d) net_d->delete_d();
  if (net_r) net_r->delete_r();
  net_r=net_d=NULL;
}
