#ifndef _NET_H_
#define _NET_H_

class net_unit;
#include "object.h"

/*
The net is a data structure used for pairs of Objects (p1 and p2 are the two parties). It's a bit like a linked list in two dimensions. By going up and down (links u & d) you traverse a list of net units involving p1 and by going left and right (links l and r) you get the list of units involving p2. Each Object has a vertical and horizontal pointer to the net (net_d and net_r). There is one net for one stage, but the stage does not have a direct pointer to it, only the Objects do.

b: is the unit a blake (if so, it's being used to solve a set of collisions)
compromise: I don't think this is being used.
collected: used by the blake routines

arg is the angle of a collision, cosarg=cos(arg), sinarg=sin(arg) 
J is the size an intended impulse (relative acceleration of one away from the other)
When an impulse takes place, p1p is the proportion of this that p1 will accelerate. p2 will accelerate the rest. It is 0-1, often 0, meaning p1 doesn't accelerate.
mu is the friction coefficient (0-1)
target_shear: when friction has fully taken affect, this should be the shear_speed (usually 0)

norm_speed: speed by which p1 & p2 are moving away from each other
shear_speed: relative sideways motion, also relative to target_shear


Useful functions:
components: calculates norm_speed and shear_speed
arg_eq: sets arg, cosarg and sinarg
new_blake: ??
vreset: calls vreset for p1 and p2, returning the objects to their original speeds
impulse: Accelerates p1 & p2 away from each other by amount J. If friction is on, shear speed is reduced.

Size of net unit - under 100B. This could cost 100s of kB in a stage with 100 objects.
*/

struct c_notification;
struct c_reporter;

typedef struct kind_pair
{
  char checks;
  void (*proc)(Object*,Object*,c_notification,c_reporter&,char);  
};

extern kind_pair kinpair[max_xkinds*max_xkinds];

class net_unit
{
public:
  Object *p1,*p2;
  kind_pair kp;
  net_unit *u,*d,*l,*r;
  char b,compromise,collected,limited;
  float arg,cosarg,sinarg;
  float J,p1p,mu,limit;
  float shear_speed,norm_speed,target_shear;

  net_unit(Object *o1,Object *o2,kind_pair kp1);

  void components();

  void arg_eq(float n_arg);

  void new_blake(float n,float m,float ts);
  void set_limit(float l);

  void vreset();

  void impulse(char friction_on);

  void delete_d();
  void delete_r();
};

#endif 
