#ifndef O_1_H_
#define O_1_H_

#include <cstdio>
#include <vector>
using namespace std;
#include "window.h"
#include "xkinds3.h"
#include "view.h"
#include "object.h"

/*
All the object kinds are declared here in alphabetical order. Being extension classes of Object, which has a lot of virtual functions, they have a lot of the same functions. These are abbreviated (see macros in object.h) in case they need to be changed. The implementations are in o_*.cpp and not in any particular order.

 --------------- Background -------------*/

struct Backg:Object
{
  int colour,pattern;

  Backg();
  SeeD; SLD;
};

/* -----------  Chain ---------------------*/

#define Chain_max_length 500

struct Chain:Object
{
  bool held,want_held,loose;
  Object *below;
  int length;
  
  Chain();
  SLD; SeeD;
 OnebodyD; EditD; ReshapeD;
  void key_Q();
  void key_A();
};


/* ---------- Control box  -----------
     definition appears at end       */
struct Ctrlbox;

/*--------------- Device-------------------*/

struct Device:Object
{
  bool on,want_on;
  Ctrlbox *mover;
};

/*---------------- Door -------------*/

struct Door:Object
{
  int colour,way;
  float way_thru;
//way: 1=top, 2=bottom, 3=left, 4=right

  Door();
  SLD; SeeD; EditD;
 ReshapeD;
};

/* ------- Electric field -----------------------*/

#define Zapper_max_length 200

struct Zapper:Device
{
  int length;
  bool vertical;
  char a1,w1,b1;

  Zapper();
  SLD; EditD; SeeD; 
ReshapeD;
  void key_Q();
  void key_O();
  void key_A();
  void key_P();
};

/* ----------- Ferris wheel ----------------*/

#define Ferris_n 6
struct Platform;

struct Ferris:Object {
  Platform *a[Ferris_n];
  float r;
  int T,phase; // r=radius, T=period

  Ferris();
  SLD; 
SeeD; OnebodyD; EditD; ReshapeD;
};

/*-------------- Gettable -----------------

The Mallet, Bat and Trumpet kinds use this struct
*/

struct Star; //forward dec
struct Gettable:Object
{
  Star *owner;
  char held;  // 0=not held, 1=held normally, 2=just taken, not to be drawn

  Gettable();
  SeeD; EditD;
  void deport();
};

/*------------ Gong ----------------*/

struct Gong:Object
{
  float th;
  int sound_time;

  Gong();
  SeeD; OnebodyD; EditD;
};

/* ---------- Heavyweight block -----------*/

struct Heavy:Object
{
  Heavy();
  OnebodyD; SeeD; 
EditD;
};

/* ----------- Home ------------------ */

struct Home:Object
{
  int colour;

  Home();
  SLD; EditD; SeeD;
};

/* ------------ Iceblock -------------*/

struct Iceblock:Object {
  Iceblock();
  EditD; SeeD;
};


/*----------  Key -------------------*/

struct Quay:Gettable
{
  int colour;  

  Quay();
  SLD;
  SeeD; EditD;
};

/* ------------ Laser ---------------*/

#define Laser_c 400
#define Laser_T 3

struct Laser:Device
{
  bool lrdir;
   
  Laser();
  SLD; EditD;
 SeeD; OnebodyD;
};


/*---------------- Lever ----------*/

#define Lever_max_amp 127

struct Lever:Object
{
  Platform *a1,*a2;
  float speed;
  int amplitude,arm_length,Platform_width;
  int colour;
  bool lhi;

  void reshape();
  Lever();
  SLD;
 EditD; OnebodyD; SeeD;
  void key_Q();
  void key_A();
  void key_O();
  void key_P();
};


/*--------------  Magnet ------------- */

#define Magnet_speed 5

struct Magnet:Device
{
  char leftmove,rightmove,phase;
   
  Magnet();
  SLD;
 EditD; OnebodyD; SeeD; MoveD;
};

/*----------------- Mirror ---------------*/

struct Mirror:Object
{
  bool broken; int way;
  float way_thru;
  int cx,cy;
// way: 1=UL, 2=UR, 3=DL, 4=DR, 5=big circle

  Mirror();
  SLD; SeeD;
 OnebodyD; EditD;
  void way_calc();
};


/*---------  Photon ------- */

struct xy_list
{ 
  int x,y; 
  xy_list *prev,*next;
  xy_list(int x1,int y1,xy_list *p);
  ~xy_list();
};

struct Photon:Object
{
  xy_list *tracef,*traceb;

  Photon();
  ~Photon();
  void add_point(int x,int y);
  OnebodyD; SeeD;
};

/* ----------------- Platform -------*/

struct Platform:Object
{
  bool impact;
  int width;

  Platform();
  SLD;
 SeeD; ReshapeD;
  void go_to(int x,int y,Object *aa);
};

/* ------------- Player ------------- */

struct Player_moveset
{
  char shiftmove,ctrlmove,leftmove,rightmove,upmove,downmove,
       ltmove,gtmove,entermove,tabmove,kpmove[10];

  void clear();
  void save_keys(ostream&,Player_moveset *old);
  void save_key(char k,ostream&);
  void operator<<(istream&);
  void check_keys();
  void check_key(char k);
};

struct Player:Object
{
  int current_Star,how_many_home,how_many_dead;
  int what_arrows,tab_time;
  vector <Star*> s;
  Star *sc;
  Object *focus,*old_focus;
  char cheerme;
  Player_moveset m;
  view vi;

  void sceq(int n);
  void add_Star(Star *s1);
  Player();
  int dsq2Star(Star *s1);
  SLD; OnebodyD; SeeD; MoveD;
};

/* ------------ Push-me Block --------- */

struct Pushme:Object
{
  bool doubled;

  Pushme();
  OnebodyD;
 EditD; SeeD;
};

/*----------- Rectangular set piece ----------*/

#define Setp2_max_height 1500
#define Setp2_max_width 1500

struct Setp2:Object
{
  int colour1;
  int width,height,pattern;

  void reshape();
  Setp2();
  SLD;
 SeeD; EditD;
  void key_Q();
  void key_A();
  void key_O();
  void key_P();
};

/* ------------ Seismos ----------- */

struct Seismos_shake
{
  Seismos_shake *next,*prev;
  float t,w,ax,ay;
};

struct Seismos:Object
{
  Seismos_shake *shakes;
  int cx,cy;

  Seismos();
  LoadD;
 EditD; OnebodyD; SeeD;
};

/* ------------ Set Piece struct (Fixed shape) ------
what=
4,5:curve
6: decoration
*/

struct Setpiece:Object
{
  int what;
  int sx,sy;

  Setpiece();
  SLD; SeeD; EditD;
};

/*------------- Spikes -----------------*/
#define max_Spikes 127

struct Spikes:Object
{
  int numb;
  bool ud_dir;

  SLD; EditD; SeeD;
 ReshapeD;
  Spikes();

  void key_P();
  void key_O();
};


/* ------------------ Star ------------*/

struct Star:Object
{
  char upmove,downmove,leftmove,rightmove,entermove,releasemove,shiftmove;

  char amihome,amifront;
  bool amidead,amiasleep;
  char sl_phase;
  float spin_cw,spin_acw,phase;
  bool floor_force,above_force;
  Object *focus;
  int colour,idle_time;

  vector <Gettable*> inv;
  int inv_ptr;

  Star();
  SLD; ReshapeD; EditD; SeeD;
 OnebodyD;

  void wake_up();
  void kill();
  bool amisquashed();
  float rolling(float arg);
};

/*------------- Teleport -------------*/

struct Teleport:Object
{
  Teleport *partner;
  char phase;
  int colour;

  Teleport();
  SLD; ReshapeD; OnebodyD; SeeD;
 EditD;
};

/*----------------- TrapDoor --------------*/

struct Door2:Device
{
  Door2();
  SLD; EditD; SeeD;
 ReshapeD;
};

/*------------ Zed ----------------*/
struct Zed:Object
{
  Zed();
  SeeD;
};

/* ---------- Appendix: Control Box ---------*/

struct Ctrlbox:Object
{
  Magnet *mag;
  Zapper *zap;
  Door2 *trap;
  Laser *laz;
  Device *dev;
  bool super;

  Ctrlbox();
  SLD; EditD;
 SeeD;
};

#endif 
