#include "str.h"
#include "o_1.h"

//---------------------------- Control box ----------------
#define CurrentType Ctrlbox

/*

20: Magnet being controlled
21: Electric field which can be turned off
22: Trapdoor which can be turned on & off
23: If it's the super button
24: Laser which can be turned on & off

*/

  Ctrlbox::Ctrlbox()
  {
    kind=xk_Ctrlbox;
    super=0;
    mag=NULL; zap=NULL; trap=NULL; laz=NULL; dev=NULL;
    sh_eq(new_point());
  }

  EditI
  {
    w->WrStr("Selected:");
    if (mag)
    {
      w->WrStr("Magnet controls");
      w->Ln();
      w->BoolEdit("On",&mag->on);
    }
    if (zap)
    {
      w->WrStr("  Electric field");
      w->WrStr("   immobiliser");
      w->Ln();
      w->BoolEdit("On",&zap->on);
    }
    if (trap)
    {
      w->WrStr(" Trapdoor lever");
      w->Ln();
      w->BoolEdit("On",&trap->on);
    }
    if (laz)
    {
      w->WrStr("  Laser lever");
      w->Ln();
      w->BoolEdit("On",&laz->on);
    }
    if (super)
    {
      w->WrStr(" Super button");
    }
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    if (f(20)) { f >> OB(mag); dev=mag; }
    if (f(21)) { f >> OB(zap); dev=zap; }
    if (f(22)) { f >> OB(trap); dev=trap; }
      super=f(23);
    if (f(24)) { f >> OB(laz); dev=laz; }
    if (dev) dev->mover=this;
  }

  SaveI
  {
    if (mag) f << 20 << mag;
    if (zap) f << 21 << zap;
    if (trap) f << 22 << trap;
    if (super) f << 23;
    if (laz) f << 24 << laz;
  }

  SeeI
  {
    if (visible(v,20+super*20,20+super*20))
    {
      int t=5;
      if (super) t=7;
      if (zap) t=5+zap->on;
      if (mag) t=2-mag->on;
      if (laz || trap) t=3+dev->on;
      v->add(4,xk_Ctrlbox,vpx,vpy,t);
    }
  }

//  DeleteI

#undef CurrentType
//--------------------------Electric field ---------------------
#define CurrentType Zapper

/*
20: length
21: is it on?
22: is it vertical?

*/
#define Zapper_max_length 200

  void Zapper::reshape()
  {
    if (vertical) sh_eq(new_rectangle(0,length));
    else sh_eq(new_hlin(length));
  }

  Zapper::Zapper()
  {
    kind=xk_Zapper;
    on=want_on=vertical=0;
    length=25;
    on=1; //not normally
    mover=NULL;
    a1=random()%5+1;
    b1=(2*length)/(random()%10+10);
    w1=random()%10+9;
  }

  EditI
  {
    w->WrStr("Selected:");
    w->WrStr(" Electric Field");
    w->Ln();
    w->NumEdit(" length: ",&length,s_int);
    w->Ln();
    w->BoolEdit("Vertical",&vertical);
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    if (f(20)) f >> length;
    if (f(21)) on=1;
    else on=0;
    if (f(22)) vertical=1;
  }

  SaveI
  {
    f << 20 << length;
    if (on) f << 21;
    if (vertical) f << 22;
  }

  SeeI
  {
    if (visible(v,length*(!vertical)+5,25*vertical+5))
      v->add(6,xk_Zapper,vpx,vpy,length,on,vertical);
  }

//  DeleteI

  void Zapper::key_Q()
  {
    length--;
    if (!length) length=1;
  }

  void Zapper::key_O() {key_Q();}

  void Zapper::key_A()
  {
    length++;
    if (length>Zapper_max_length) length--;
  }

  void Zapper::key_P() {key_A();}

#undef CurrentType
//----------------------------Laser----------------------
#define CurrentType Laser


/*
            Laser class
20: on
21: pointing right?

*/


  Laser::Laser()
  {
    kind=xk_Laser;
    on=want_on=lrdir=0;
    sh=new_rectangle(15,7);
  }

  EditI
  {
    w->WrStr("Selected: Laser");
    w->Ln();
    w->BoolEdit("On",&on);
    w->Ln();
    w->WrStr("Pointing:");
    w->GiveChoices(l_or_r,2,&lrdir,1);
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    on=f(20);
    lrdir=f(21);
  }

  SaveI
  {
    if (on) f << 20;
    if (lrdir) f << 21;
  }

  OnebodyI
  {
    Object *m;

    if (on) {
      m=new_x(xk_Photon);
      m->togo=Laser_T;
      locate_with_me(m);
      if (lrdir) m->vx=Laser_c;
      else      m->vx=-Laser_c;
      m->calibrate();
      cr << m;
    }
  }

  SeeI
  {
    if (visible(v,15,10)) v->add(4,xk_Laser,vpx,vpy,lrdir);
  }

#undef CurrentType
//------------------------------ Magnet ----------------
#define CurrentType Magnet
/*
20: on
21: do we want it on?

*/

  Magnet::Magnet()
  {
    kind=xk_Magnet;
    on=want_on=0;
    leftmove=rightmove=0;
    phase=0;
    sh=new_rectangle(15,15);
  }

//  DeleteI

  EditI
  {
    w->WrStr("Selected: Magnet");
    w->Ln();
    w->BoolEdit("On",&on);
    w->BoolEdit("Wanted On",&want_on);
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    on=f(20);
    want_on=f(21);
  }

  SaveI
  {
    if (on) f << 20;
    if (want_on) f << 21;
  }

  OnebodyI
  {
    Object *m;
    float tvx;

    phase=(phase+1)%20;

    tvx=0;
    if (leftmove)  tvx-=Magnet_speed;
    if (rightmove) tvx+=Magnet_speed;
    vx=vx*0.6+tvx*0.4;

    if (on)
    {
      m=new_x(xk_Magneton);
      m->togo=4;
      locate_with_me(m);
      m->vy=300; m->calibrate();
      cr << m;
    }
  }

  SeeI
  {
    if (visible(v,15,15))
      v->add(4,xk_Magnet,vpx,vpy,on && phase<10);
  }

  MoveI
  {
    leftmove=leftmove/2;
    rightmove=rightmove/2;
  }

#undef CurrentType
//----------------------------- TrapDoor -------------------
#define CurrentType Door2
/*

20: on
21: do we want it on?

*/

  void Door2::reshape()
  {
    if (on) sh_eq(new_rectangle(20,10));
    else sh_eq(NULL);
  }

  Door2::Door2()
  {
    kind=xk_Door2;
    want_on=0; on=1; reshape();
  }

  SeeI
  {
    if (on && visible(v,20,10)) v->add(3,xk_Door2,vpx,vpy);
  }

//  DeleteI

  EditI
  {
    w->WrStr("Selected:");
    w->WrStr("  TrapDoor");
    w->Ln();
    w->BoolEdit("On",&on);
    w->BoolEdit("Wanted On",&want_on);
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    on=f(20);
    want_on=f(21);
  }

  SaveI
  {
    if (on) f << 20;
    if (want_on) f << 21;
  }

#undef CurrentType
// --------------------- Photon ------------------------
#define CurrentType Photon

  xy_list::xy_list(int x1,int y1,xy_list *p)
  { x=x1; y=y1; prev=p; next=NULL;  }

  xy_list::~xy_list()
  {  delete next;   }

  Photon::Photon()
  {
    kind=xk_Photon;
    tracef=traceb=NULL;
    sh_eq(new_point());
  }

  Photon::~Photon()
  { 
    delete tracef;
  }

//  DeleteI

  void Photon::add_point(int x,int y)
  {
    traceb=new xy_list(x,y,traceb);
    if (!tracef) tracef=traceb;
    else traceb->prev->next=traceb;
  }

  OnebodyI
  {
    delete tracef;
    tracef=traceb=NULL;
    add_point(px,py);
  }

// This see thing here could do with some correction.

  SeeI
  {
    xy_list *l1,*l2;
    for (l1=tracef; l1; l1=l2)
    {
      int x0,x1,y0,y1;
      x0=l1->x-v->px;  y0=l1->y-v->py;

      if ((l2=l1->next))
      {
        x1=l2->x-v->px;  y1=l2->y-v->py;
      }
      else { x1=px-v->px; y1=py-v->py; }

      if ((x0==x1 && y0==y1) || (x0>v->sx && x1>v->sx) || (x0<-v->sx && x1<-v->sx) || (y0>v->sy && y1>v->sy) || (y0<-v->sy && y1<-v->sy)) {}

      else v->add(5,xk_Photon,x0,y0,x1,y1);
    }
  }

#undef CurrentType
//--------------------------------- Mirror ----------------------
#define CurrentType Mirror

/*
20: broken
21: way: 1=UL, 2=UR, 3=DL, 4=DR, 5=big circle

*/

int Mirror_triangle1[]={ -10,10,  10,-10, 10,10};
int Mirror_triangle2[]={-10,-10,  10,10, -10,10};
int Mirror_triangle3[]={-10,-10, 10,-10,  10,10};
int Mirror_triangle4[]={-10,-10, 10,-10, -10,10};

  Mirror::Mirror()
  {
    kind=xk_Mirror;
    broken=0; way=1;
  }

  SeeI
  {
    if (visible (v,10+30*(way==5),10+30*(way==5)))
      v->add(5,xk_Mirror,vpx,vpy,way-1,broken);
  }

//  DeleteI

  void Mirror::way_calc()
  {
    if (way==1)
    {
      way_thru=-3*PI/4;
      sh_eq(make_polygon(3,Mirror_triangle1));
    }
    if (way==2)
    {
      way_thru=-PI/4;
      sh_eq(make_polygon(3,Mirror_triangle2));
    }
    if (way==3)
    {
      way_thru=3*PI/4;
      sh_eq(make_polygon(3,Mirror_triangle3));
    }
    if (way==4)
    {
      way_thru=PI/4;
      sh_eq(make_polygon(3,Mirror_triangle4));
    }
    if (way==5)
      sh_eq(new_circl(40));
  }

  OnebodyI
  {
    if (way==5)
    {
      if (vy==0) vy=1;
      if (py>cy+20) vy=-1;
      if (py<cy-20) vy=1;
      if (key[KEY_SPACE]) vy=0;
      if (key[KEY_Z]) vy=1;
      if (key[KEY_A]) vy=-1;
    }
  }

  EditI
  {
    if (way==5)
      w->WrStr("Disco Ball!");
    else
    {
      w->WrStr("Selected: Mirror");
      w->Ln();
      w->WrStr("Facing:");
//      w->GiveChoices(way2_choices,4,&way,2);
    }
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    broken=f(20);
    if (f(21)) f >> way;
    way_calc(); cx=px; cy=py;
  }

  SaveI
  {
    if (broken) f << 20;
    if (way!=1) f << 21 << way;
  }

#undef CurrentType

