#include <cmath>
#include "str.h"
#include "o_1.h"

// -------------------------------- Platform ---------------------
#define CurrentType Platform

/*
20: width
later: colour/pattern
*/

  ReshapeI
  {   sh_eq(new_hlin(width)); }

  Platform::Platform()
  {
    kind=xk_Platform;
    width=15;
    impact=0;
    reshape();
  }

  SaveI
  {   f << 20 << width; }

  LoadI
  {  if (f(20)) f >> width; }

  SeeI
  {  if (visible(v,width,10)) v->add(3,xk_Platform,vpx,vpy); }

  void Platform::go_to(int x,int y,Object *aa)
  {
    vx=x-px; vy=y-py;
    calibrate();
  }

#undef CurrentType
// ----------------------------- Lever -------------------------
#define CurrentType Lever

#define Lever_max_amp 127

  ReshapeI
  {
    a1->px=px-arm_length;
    a2->px=px+arm_length;
    a1->py=py-amplitude*(2*lhi-1);
    a2->py=py+amplitude*(2*lhi-1);
    sh_eq(new_rectangle(arm_length,amplitude));
  }

  Lever::Lever()
  {
    kind=xk_Lever;
    amplitude=20;
    arm_length=28;
    Platform_width=20;
    lhi=0;
    speed=20;
  }

  EditI
  {
    w->WrStr("Selected: Lever");
    w->Ln();
    w->NumEdit("Amplitude : ",&amplitude,s_int);
    w->NumEdit("Arm length: ",&arm_length,s_int);
    w->Ln();
    w->NumEdit("Speed: ",&speed,s_float);
    w->Ln();
    w->NumEdit("Colour: ",&colour,s_colour);
    w->Ln();
    w->GiveChoices(lhi_or_not,2,&lhi,1);
    w->Ln();
    WrPos(w);
  }

  LoadI
  {
    if (f(20)) f >> OB(a1);
    if (f(21)) f >> OB(a2);
    if (f(22)) f >> speed;
    lhi=f(23);
    if (f(24)) f >> amplitude;
    if (f(25)) f >> arm_length;
    if (f(26)) f >> colour;
  }

  SaveI
  {
    if (a1) f << 20 << a1;
    if (a2) f << 21 << a2;
    f << 22 << speed;
    if (lhi) f << 23;
    f << 24 << amplitude << 25 << arm_length << 26 << colour;
  }

  OnebodyI
  {
    if (lhi && a1->impact)
    {
      lhi=0;
      a1->vy=speed;
      a2->vy=-speed;
      a1->calibrate();
      a2->calibrate();
    }
    else if (!lhi && a2->impact)
    {
      lhi=1;
      a1->vy=-speed;
      a2->vy=speed;
      a1->calibrate();
      a2->calibrate();
    }
    a1->impact=0; a2->impact=0;
  }

  SeeI
  {
    if (visible(v,arm_length,amplitude)) 
      v->add(6,xk_Lever,vpx,vpy,arm_length,a1->py-py,colour);
  }

  void Lever::key_Q()
  {
    amplitude--;
    if (!amplitude) amplitude=1;
  }

  void Lever::key_A()
  {
    amplitude++;
    if (amplitude>Lever_max_amp) amplitude=Lever_max_amp;
  }

  void Lever::key_O()
  {
    arm_length--;
    if (!arm_length) arm_length=1;
  }

  void Lever::key_P()
  {
    arm_length++;
    if (arm_length>Lever_max_amp) arm_length=Lever_max_amp;
  }

#undef CurrentType
// ----------------------------- Ferris wheel -----------------------
#define CurrentType Ferris

  ReshapeI
  {
    int j;
    float q;
    for(j=0;j<Ferris_n;j++)
    {
      q=PI/3*j;
      locate_with_me(a[j]);
      a[j]->move((int) (r*cos(q)), (int) (r*sin(q)));
    }
  }

  Ferris::Ferris()
  {
    kind=xk_Ferris;
    r=100; T=140; phase=0;
  }

  LoadI
  {
    if (f(20))
      for(int j=0;j<Ferris_n;j++)
        f >> OB(a[j]);
  }

  SaveI
  {
    f << 20;
    for (int j=0;j<Ferris_n;j++)
      f << a[j];
  }

  EditI
  {
    w->WrStr("Ferris wheel");
    w->Ln();
    WrPos(w);
  }

  OnebodyI
  {
    int j; float q;
    phase++;
    if (phase==T) phase=0;
    for (j=0;j<Ferris_n;j++)
    {
      q=PI/3*((float)j)+PI*2*((float)phase)/((float)T);
      a[j]->go_to(px+fl2i(r*cos(q)),py+fl2i(r*sin(q)),a[j]); //last parameter useless
    }
  }

  SeeI
  {
    if (visible(v, (int)r, (int)r))
      v->add(9,xk_Ferris,vpx,vpy,a[0]->px-px,a[0]->py-py,a[1]->px-px,a[1]->py-py,a[2]->px-px,a[2]->py-py);
  }

#undef CurrentType
