#include <cmath>
#include <allegro.h> // for keyboard stuff
#include "basics.h"
#include "o_1.h"

#include <iostream>

// ---------------------------------- Star ------------------------
#define CurrentType Star

/*
20: lrdir
21: colour
22: inventory
*/

  Star::Star()
  {
    kind=xk_Star;
    colour=1; sl_phase=0;
    phase=0.1;
    inv_ptr=0;
    amihome=amifront=amidead=amiasleep=0;
    spin_acw=spin_cw=0;
    sh_eq(new_circl(15));
    focus=this;
  }

  EditI
  {
    w->WrStr("Selected: Star");
    w->Ln();
    w->WrStr("Colour:");
    w->GiveChoices(colour_choices,4,&colour,1);
    w->Ln();
    w->BoolEdit("Sleeping",&amiasleep);
    w->Ln();
    WrPos(w);
  }

  SaveI
  {
    if (colour!=1) f << 21 << colour;
    if (inv.size())
    {
      f << 22 << (int)inv.size() << inv_ptr;
      for(int j=0;j<(int)inv.size();j++)
        f << inv[j];
    }
    if (amiasleep) f << 23;
  }

  LoadI
  {
    Gettable *g;
    int n;

    f(20);
    if (f(21)) f >> colour;
    if (f(22))
    {
      f >> n >> inv_ptr;
      while (n--)
      {
        f >> OB(g);
        inv.push_back(g);
        g->owner=this;
      }
    }
    amiasleep=f(23);
  }

  ReshapeI{}

  OnebodyI
  {
    Gettable *g;
    Object *f;
    int d1,d2;
    float q;

  //  if (togo==35) { cr << Sfx(sfx_Bell); }
    if (amidead) focus=NULL; else focus=this;
    floor_force=above_force=0;

    if (!amifront && (Abs(vx)+Abs(vy)<0.5))
      idle_time++;
    else idle_time=0;

    if (idle_time>60)
      if (phase>0.5) spin_cw=0.025;
      else if (phase>0.025) spin_acw=0.025;
    phase=phase+spin_cw-spin_acw+1;
    phase-=(int)phase;
    spin_cw=spin_acw=0;
    if (releasemove && inv.size())
    {
      g=inv[inv_ptr];
      locate_with_me(g);
      if (releasemove<4) g->vy+=3;
      if (releasemove>6) g->vy-=3;
      if (releasemove%3==0) g->vx+=3;
      if (releasemove%3==1) g->vx-=3;
      d1=(int)Abs(g->vx); d2=(int)Abs(g->vy);
      g->togo=0x10000+min2(13,100/(1+2*(d1+d2)-min2(d1,d2)));
      g->held=0;
      for(int j=inv_ptr;j<(int)inv.size()-1;j++)
        inv[j]=inv[j+1];
      inv.pop_back();
      if (inv_ptr && inv_ptr==(int)inv.size()) inv_ptr--;
      cr << g;
      
      if (releasemove==5 && g->kind==xk_Trumpet)
      {
        for(q=0;q<2*PI;q+=0.1)
        {
          f=new_x(xk_Phonon);
          locate_with_me(f);
          f->togo=4;
          f->vx+=(float)100*cos(q);
          f->vy+=(float)100*sin(q);
          f->calibrate();
          cr << f;
        }
        g->togo=80+0x10000;
        cr << Sfx(sfx_Trumpet);
      }
    }
    if (amihome) amihome--;
    if (amiasleep)
    {
      sl_phase=(sl_phase+1)%19;
      if (sl_phase==17)
      {
        f=new_x(xk_Zed);
        locate_with_me(f);
        f->vx+=1; f->togo=70;
        f->calibrate();
        cr << f;
      }
    }

    vy+=1.7; //gravity
  }

  SeeI
  {
    if (visible(v,15,15))
    {
      int ph;
      if (amiasleep) ph=30; else ph=(int)(phase*40);
	if (ph<0 || ph>39) {
	//      orep << "Star ph=" << ph << endl;
	      ph = 0;	//Fixes wrong phase of star
	}
      v->add(5,xk_Star,vpx,vpy,colour-1+amifront*4,ph);
    }
  }

  void Star::wake_up()
  {
    if (amiasleep)
    {
      amiasleep=0;
      idle_time=61;
    }
  }

  void Star::kill() 
  {
    amidead=1;
    wake_up();
    togo=50;
  }

//  DeleteI

  bool Star::amisquashed()
  {
    return floor_force && above_force;
  }

  float Star::rolling(float arg)
  {
    char w;
    float r;
    if (shiftmove) return 0;
        if (arg>=-3*PI/4 && arg<=-PI/4) w=1; // floor
    else if (arg<=3*PI/4 && arg>=PI/4)  w=2; // ceiling
    else if (Abs(arg)>PI/2)             w=3; // wall on right;
    else                                w=4; // wall on left;
    r=0;
    if ((leftmove && w==1)||(rightmove && w==2)||(downmove && w==3)||(upmove && w==4))
      { r+=5; spin_acw=0.05; }
    if ((rightmove && w==1)||(leftmove && w==2)||(upmove && w==3)||(downmove && w==4))
      { r-=5; spin_cw=0.05; }
    return r;
  }

#undef CurrentType
// ------------------------------------ Player ---------------
#define CurrentType Player

  void Player_moveset::clear()
  {
    int j;
    entermove=tabmove=gtmove=ltmove=0;
    for(j=1;j<10;j++)
      kpmove[j]=0;
    downmove=ctrlmove=shiftmove=upmove=rightmove=leftmove=0;
  }

  void Player_moveset::save_keys(ostream &r,Player_moveset *old)
  {
    for(int j=1;j<10;j++)
      if (kpmove[j]&1)             r << j << " ";
    if (leftmove !=old->leftmove)  r << 11 << " ";
    if (rightmove!=old->rightmove) r << 12 << " ";
    if (upmove   !=old->upmove)    r << 13 << " ";
    if (downmove !=old->downmove)  r << 14 << " ";
    if (shiftmove!=old->shiftmove) r << 15 << " ";
    if (ctrlmove !=old->ctrlmove)  r << 16 << " ";
    r << 0 << " ";
  }

  void Player_moveset::save_key(char k,ostream &r)
  {
    if (k==KEY_TAB)   r << 20 << " ";
    if (k==KEY_ENTER) r << 19 << " ";
    if (k==KEY_COMMA) r << 18 << " ";
    if (k==KEY_STOP)  r << 17 << " ";
  }

  void Player_moveset::operator<<(istream &r)
  {
    int n;

    for(int j=1;j<10;j++) kpmove[j]*=2;

    entermove=tabmove=gtmove=ltmove=0;
    r >> n;
    while (n)
    {
      if (n<10) kpmove[n]++;
      if (n==11) leftmove ^=255;
      if (n==12) rightmove^=255;
      if (n==13) upmove   ^=255;
      if (n==14) downmove ^=255;
      if (n==15) shiftmove^=255;
      if (n==16) ctrlmove ^=255;
      if (n==17) gtmove++;
      if (n==18) ltmove++;
      if (n==19) entermove=1;
      if (n==20)
        if (shiftmove)tabmove--;
        else          tabmove++;
      r >> n;
    }
  }

  void Player_moveset::check_keys()
  {
    int j;
    entermove=tabmove=gtmove=ltmove=0;

    leftmove=key[KEY_LEFT];
    rightmove=key[KEY_RIGHT];
    upmove=key[KEY_UP];
    downmove=key[KEY_DOWN];
    ctrlmove=keyCTRL;
    shiftmove=keySHIFT;

    for(j=1;j<10;j++)
    {
      kpmove[j]*=2;
      if (key[KEY_0_PAD+j])
        kpmove[j]++;
    }
  }

  void Player_moveset::check_key(char k)
  {
    if (k==KEY_TAB)
      if (keySHIFT) tabmove--;
      else            tabmove++;
    if (k==KEY_ENTER) entermove=1;
    if (k==KEY_COMMA) ltmove=1;
    if (k==KEY_STOP)  gtmove=1;
  }


/*  start of player struct

Player record in file:

20: number of stars (0) followed by descriptions of stars
21: current star    (0)

*/
  void Player::sceq(int n)
  {
    if (n>=(int)s.size())
    {
      orep << "sceq " << n << endl;
      exit(0);
    }
    current_Star=n; sc=s[n];
  }

  Player::Player()
  {
    kind=xk_Player;
    current_Star=how_many_dead=0; //will we load a level with casualties already?
    sc=NULL; focus=old_focus=this;
    cheerme=0;
  }

//  DeleteI

  int Player::dsq2Star(Star *s1)
  {
    return sq(s1->px-px)+sq(s1->py-py);
  }

  LoadI
  {
    int n;
    if (f(20))
    {
      f >> n;
      while (n--)
      {
        f >> OB(sc);
        s.push_back(sc);
      }
    }
    if (f(21)) { f >> n; sceq(n); }
    if (f(22)) f >> how_many_dead;
  }

  SaveI
  {
    if (!s.empty())
    {
      f << 20 << (int)s.size();
      for (int j=0; j<(int)s.size(); j++)
        f << s[j];
    }
    if (how_many_dead) f << 22 << how_many_dead;
  }

  OnebodyI
  {
    int j,tpx,tpy;
    float tvx,tvy;
    Star *dummy;

    vi.clear();
    vi.sx=320;vi.sy=220;
    vi.px=px;vi.py=py;
    j=how_many_home=0;
    for (j=0; j<(int)s.size(); )
    {
      if (s[j]->amidead)
      {
        how_many_dead++;
	if(s.size()>1) {		//replace dead star with lastone
		dummy = s[j];
	        s[j]=s[s.size()-1];	
		s[s.size()-1] = dummy;
	}
	s.pop_back();			//remove last star
        if (j==current_Star) m.ctrlmove=1;
	return;
      }
      else
      {
        if (s[j]->amihome) how_many_home++;
        j++;
      }
    }

    if (old_focus!=focus) tab_time=30;
    if (tab_time) tab_time--;
    old_focus=focus;

    if (focus==this)    what_arrows=0; // Bracket for sc, Cross-hair for player
    else if (focus==sc) what_arrows=2; // Arrow   for sc
    else                what_arrows=4; // Bracket for sc, Arrow for device
    what_arrows+=(tab_time%10>4);      // arrow flash

    if (s.empty()) return;
     // need to make sure one is selected

    for(j=0; j<(int)s.size(); j++) s[j]->amifront=(j==current_Star);

    if (!m.ctrlmove && sc)
    {
	    if(focus) {
		tpx=focus->px-px;tpy=focus->py-py;
		tvx=focus->vx-vx;tvy=focus->vy-vy;
	}

      vx+=smove(tpx,tvx,3.8);
      vy+=smove(tpy,tvy,3.8);
    }
    else
    {
      vx=vx*0.8;vy=vy*0.9;
      if (m.leftmove) vx-=2.2;
      if (m.rightmove) vx+=2.2;
      if (m.upmove) vy-=2.2;
      if (m.downmove) vy+=2.2;

      if (!sc) sceq(0);
      for(j=0; j<(int)s.size(); j++)
        if (dsq2Star(sc)>dsq2Star(s[j])) {
          sceq(j);
	}
    }
    if (cheerme)
    {
      cheerme=0;
      cr << Sfx(sfx_Clap);
    }
  }

  SeeI
  {
    if (sc->kind!=xk_Star) sceq(0);
    if (sc->kind!=xk_Star) return;

    int te1,te2,j;

    if (&vi!=v) return;
    if (s.empty()) return;

    if(focus) {
	    focus->visible(v,0,0);
    }
    sc->visible(v,0,0);

    if(focus) {
	    v->add(11,xk_Player,sc->vpx,sc->vpy,sc->colour-1,sc->amiasleep, what_arrows,focus->vpx,focus->vpy, s.size(),how_many_home,how_many_dead);
    }

    te1=v->px; te2=v->py;
    v->px=v->py=0;
    for (j=0;j<(int)sc->inv.size();j++)
    {
      sc->inv[j]->see(v);
    }
    v->px=te1; v->py=te2;
  }

  MoveI
  {
    int j;

    if (sc && !m.ctrlmove) focus=sc->focus;
    else focus=this;

    if (s.empty() || !sc) return;

	while (m.ltmove--) {
		if (sc->inv_ptr>0) {
        		sc->inv_ptr--;
		}
	}
	
	while (m.gtmove--) {
		if (sc->inv_ptr<(int)sc->inv.size()-1) {
			sc->inv_ptr++;
		}
	}

    sceq((current_Star+m.tabmove+s.size())%s.size());

    for (j=0; j<(int)s.size();j++)
    {
      s[j]->upmove=0;
      s[j]->downmove=0;
      s[j]->leftmove=0;
      s[j]->rightmove=0;
      s[j]->entermove=0;
      s[j]->releasemove=0;
      s[j]->shiftmove=0;
    }
    if (!m.ctrlmove && !sc->amiasleep)
    {
      sc->leftmove=m.leftmove;
      sc->rightmove=m.rightmove;
      sc->upmove=m.upmove;
      sc->downmove=m.downmove;
      sc->shiftmove=m.shiftmove;
      sc->entermove=m.entermove;
      for (j=1;j<10;j++)
        if ((m.kpmove[j]&3)==1)
          sc->releasemove=j;
    }

    ForVec(sc->inv,Gettable *g,i)
      g->held=1;
      g->px=20*(i-sc->inv_ptr);
      g->py=0;
    }
  }


#undef CurrentType


