#ifndef _SHAPES_H_
#define _SHAPES_H_

#define se_point 1
#define se_curve 2
#define se_line  3

struct shape_elmt
{
  char kind;
  int x,y,r;
  float arg1,arg2;
  shape_elmt *prev,*next;

  shape_elmt();
  ~shape_elmt();

  bool on_line(int x1,int y1);
  bool in_line(int x1,int y1);
  float on_curve(int x1,int y1);
  bool in_curve(int x1,int y1);
  bool in_chord(int x1,int y1);
  void invert();
  void describe();
};


extern int n_shapes;

#define max_shapes 80
enum 
{ sh_noshape, sh_point, sh_circl, sh_rectangle, sh_roundrect, sh_hlin, sh_polystroke, sh_twoshapes, sh_translation};

struct shape
{
  char kind;
  int r,lr,ud;
  int max_x,max_y,min_x,min_y;
  shape_elmt *strokes;
  int n_strokes;
  shape *a1,*a2;

  shape(int k);
  ~shape();
  void calc_maxima();
  shape_elmt *find_se(float arg,bool eq);
  void add_se(int x,int y,int r,float arg1,float arg2);
  void make_strokes();
  void describe();
  bool point_in(int x,int y);
  float argument(int x,int y);
  shape *copy();                 // returns a copy of the shape
  shape *copy_opp();             // returns a laterally inverted copy
};

shape *new_point();              // Shape creation functions
shape *new_circl(int r);
shape *new_rectangle(int lr,int ud);
shape *new_roundrect(int lr,int ud,int ra);
shape *new_hlin(int lr);
shape *new_twoshapes(shape *s1,shape *s2);
shape *make_polygon(int n,int *v);
shape *translate(shape *s, int x, int y);

shape *combine_polystrokes(shape *s1,shape *s2);
shape *combine_sh(shape *s1,shape *s2);


#endif 

