#include "basics.h"
#include "fio3.h"
#include "view.h"
#include "net.h"
#include "stage.h"

  Stage::Stage()
  {
    list=NULL;
    header="Puzzle Stars Stage";
    WorkingMB.owner=this;
    Rep.owner=this;
  }

  Stage::~Stage()
  {
    list=NULL;
  }

  void Stage::subscribe(MsgQueue *mq)
  {
    subscribers.push_back(mq);
  }

  void Stage::broadcast(message &m)
  {
    for (int j=0;j<(int)subscribers.size();j++)
      subscribers[j]->q.push(m);
  }

  void Stage::add_Object(Object *x)
  {
    int k1,k2,g1,g2,g3,j;
    Object *m;
    kind_pair kp;

    if (!x) return;
    add_Object(x->next);

    for(m=list;m;m=m->next)
    {
      g1=g2=0;
      do {
        k1=m->kind;
        k2=x->kind;
        if (k1<0 || k2<0 || k1>=max_xkinds || k2>=max_xkinds)
        {
          orep << "k1="<<k1<<" k2="<<k2<<" This is unacceptable\n";
          exit(0);
        }
        for(j=0;j<g1;j++) k1=lorac[k1].xk_parent;
        for(j=0;j<g2;j++) k2=lorac[k2].xk_parent;
        kp=kinpair[k1*max_xkinds+k2];
        if (kp.checks)
        { new net_unit(m,x,kp);
          break;
        }
        else {
         kp=kinpair[k2*max_xkinds+k1];
          if (kp.checks)
          { new net_unit(x,m,kp);
            break;
          }
        }
        if (!g1) {g1=g2/2+1;g2=(g2+1)/2; }
        else if (g1>g2) {g3=g1; g1=g2; g2=g3; }
        else if (g1==g2) {g1++;g2--; }
        else {g3=g1; g1=g2+1; g2=g3-1; }
      } while (k1 || k2);
    }

    x->prev=NULL;
    x->next=list;
    if (list)
      list->prev=x;
    list=x;
  }

  void Stage::en_waiting_list(Object *x)
  {
    if (!x) return;
    en_waiting_list(x->next);

    x->next=waiting_list;
    waiting_list=x;
  }

  void Stage::remove_Object(Object *x)
  {
    x->de_net();

    if (x->next)
      x->next->prev=x->prev;
    if (x->prev)
      x->prev->next=x->next;
    else list=x->next;
  }

  void Stage::remove_list(Object *x,bool r)
  {
    if (x)
    {
      remove_list(x->next,r);
      if (r) remove_Object(x);
      delete x;
    }
  }

  void Stage::delete_list()
  { remove_list(list,true);}

  int Stage::length(Object *x)
  {
    if (!x) return 0;
    return x->saveable()+length(x->next);
  }

  void Stage::load(string fn)
  {
    delete_list();
    Object *x; int n;

    loader l(fn.c_str(),header);
    l >> n;
    while (n--)
    {
      l >> x;
      add_Object(x);
    }
    l.complete();
    for (x=list;x;x=x->next) x->reshape();
  }

  void Stage::save(string fn)
  {
    saver s(fn.c_str(),header);
    s << length(list);
    for (Object* m=list;m;m=m->next)
      if (m->saveable())
        s << m;
    s.complete();
  }

  void Stage::do_intelg_list()
  { Object *m;
    for(m=list;m;m=m->next)
      m->move();
  }

  void Stage::do_draw_list(view *v)
  {
    for (Object* m=list;m;m=m->next)
      m->see(v);
  }
