#include <allegro.h>

#include "basics.h"
#include "c_solve.h"
#include "xkinds3.h"
#include "o_1.h"
#include "stage.h"

// These macros should do the trick

#define TwoBody(Id) void Id(Object *p1,Object *p2,c_notification cn,c_reporter &cr,char v)
#define MB cr.Blatrix()

/*  Two body interaction routines
-------------------------------------------------------------------
Null */


TwoBody(d_Object_Object)
{
  orep << "xkinds2 0-0";
}


/*  -----------------------------------------
Magnetism of Star, Pushme or Heavyweight (unsuccessful) */

TwoBody(d_Liftable_Magneton)
{
  if (p2->togo==1) return;
  if (p1->kind==xk_Star)
    if (((Star*)p1)->amidead) return;
  p2->togo=1;
  if (p1->kind==xk_Heavy)
    p1->vy-=1.6;
  else
  {
    p1->vy-=2.7;
    p1->vx=p2->vx*0.2+p1->vx*0.8;
  }
  MB.consider(p1);
}


/* ------------------------------------------------
     Chains: Constraining their below object and cutting them */

TwoBody(d_Chain_Below)
{
  Chain *c=(Chain*)p1;
  if (!v)
  {
    char &x=cn.nu->kp.checks;

    if (c->below==p2 && !c->loose) x=17;
    else x=1;
  }

  if (v==4)
  {
    if (c->held)
      MB.go(cn,0,0,0);
    else if (c->vy>p2->vy) 
      c->loose=1;
    else
     MB.go(cn,1,0,0);
  }
}

TwoBody(d_Chain_Cutter)
{
  Chain *c1=(Chain*)p1;
  if (c1->py>p2->py) return;

  p2->togo=1;
  Chain *c3=new Chain;
  c3->py=p2->py;
  c3->px=c1->px;
  c3->held=0;
  c3->below=c1->below;
  c1->below=NULL;
  c1->length=p2->py-c1->py;
  cr << Sfx(sfx_ChainCut);
  cr << c3;
}




/* -------------------------------------------------------------
 Gettable usage: Mirror smashing & Door opening */

TwoBody(d_Mirror_Bat)
{
  Mirror *m=(Mirror*)p1;

  if (argeq(cn.arg,m->way_thru)&& !m->broken)
  {
    m->broken=1;p2->togo=1;
    cr << Sfx(sfx_MirrorSmash);
  }
  else
    MB.go(cn,0,0.2,0);
}

TwoBody(d_Door_Quay)
{
  Door *d=(Door*)p1;
  Quay *q=(Quay*)p2;

  if ((d->colour==q->colour) && argeq(cn.arg,d->way_thru))
  {
    d->togo=1;q->togo=1;
    cr << Sfx(sfx_DoorOpen);
  }
  else
  {
    MB.go(cn,0,0.2,0);
    cr << Sfx(sfx_Buzz);
  }
}


/* --------------------------------------------
Noise: Mallet hits gong and Star is woken up by phonon */

TwoBody(d_Gong_Mallet)
{
  ((Gong*)p1)->sound_time=100;
  cr << Sfx(sfx_Gong);
}

TwoBody(d_Star_Phonon)
{
  Star *s=(Star*)p1;
  if (s->amiasleep)
  {
    p2->togo=1;
    s->wake_up();
  }
}


/* ----------------------------------------------------------
 Death of Star by spikes and electric field */

TwoBody(d_Star_Spikes)
{
  Star *s=(Star*)p1;

  if (!s->amidead)
  {
    s->kill();
    s->vy=p2->vy-13;
    cr << Sfx(sfx_StarSpike);
  }
}

TwoBody(d_Star_Zapper)
{
  Star *s=(Star*)p1;
  Zapper *z=(Zapper*)p2;
  if (!s->amidead && z->on)
  {
    s->kill();
    cr << Sfx(sfx_Zap);
  }
}

TwoBody(d_Star_Photon)
{
  Star *s=(Star*)p1;
  if (!s->amidead)
  {
    s->kill();
    p2->togo=2;
    p2->vx=p2->vy=0;
    cr << Sfx(sfx_Zap);
  }
}




/*------------------------------------------------------
     Laser destroyed by Heavyweight or by other laserbeam.
*/

TwoBody(d_Laser_Heavy)
{
  Laser *l=(Laser*)p1;
  l->togo=1;
  l->mover->togo=1;
  p2->vy=p1->vy;p2->vx=p2->vx;
  cr + p2;
  cr << Sfx(sfx_LaserSquash);
}

TwoBody(d_Laser_Photon)
{
  if (Abs(p2->vx)+Abs(p2->vy)>Laser_c/2)
  {
    Laser *l=(Laser*)p1;
    l->togo=1;
    l->mover->togo=1;
    p2->togo=1;
    p2->vy=p2->vx=0;
    cr << Sfx(sfx_Zap);
    cr + p2;
  }
}




/* ----------------------------------------------------
      Operations of lever, teleport and control box.
   Teleportation still has no collision detection. */

TwoBody(d_Star_Teleport)
{
  net_unit *n;
  char disallow=0;
  int spx,spy;
  float svx,svy;
  Star *s=(Star*)p1;
  Teleport *t1=(Teleport*)p2;
  Teleport *t2=t1->partner;

  if (s->entermove)
  {
    s->entermove=0;
    spx=s->px; spy=s->py; svx=s->vx; svy=s->vy;
    s->px=t2->px;
    s->py=t2->py;
    s->vx=t2->vx;
    s->vy=t2->vy;

    for(n=s->net_d;n;n=n->d)
      if (n->kp.checks&4)
        if (touching(n->p1,n->p2))
          disallow=1;
    for(n=s->net_r;n;n=n->r)
      if (n->kp.checks&4)
        if (touching(n->p1,n->p2))
          disallow=1;

    if (disallow)
    {
      s->px=spx; s->py=spy; s->vx=svx; s->vy=svy;
      cr << Sfx(sfx_Buzz);
    }
    else
      cr + s << Sfx(sfx_Teleport);
  }
}

TwoBody(d_Lever_Platform)
{
  Platform *p=(Platform*)p2;

  MB.go(cn,0,0,0);
  p->vy=0;
}

TwoBody(d_Star_Ctrlbox)
{
  bool on;
  Star *s=(Star*)p1;
  Ctrlbox *c=(Ctrlbox*)p2;

  if (c->dev) on=c->dev->on;
  else on=false;

  if (s->entermove && c->zap) c->zap->on=0;

  if (s->entermove && c->super)
  {
    c->super=0;
    cr << Sfx(sfx_SuperButton);
    return;
  }
  if (s->entermove && c->dev && !c->zap)  c->dev->on=!c->dev->on;

  if (s->shiftmove && c->dev)
  {
    s->focus=c->dev;

    if (c->mag)
    {
      if (s->leftmove) c->mag->leftmove=2;
      if (s->rightmove)c->mag->rightmove=2;
    }

    if (!c->zap)
    {
      if (s->upmove)   c->dev->on=1;
      if (s->downmove) c->dev->on=0;
    }
  }
  if (c->trap) c->trap->reshape();

  if (c->dev)
    if (c->dev->on!=on)
      cr << Sfx(sfx_DeviceClick);
}



/*  -----------------------------------------------
     Star collects Gettable, Star at Home */

TwoBody(d_Star_Gettable)
{
  Star *s=(Star*)p1;
  Gettable *g=(Gettable*)p2;

  if (g->owner!=s && !s->amiasleep)
  {
    g->owner=s;
    g->held=2;
    g->togo=0x10001;
    cr << Sfx(sfx_GetItem);
  }
}

TwoBody(d_Star_Home)
{
  Star *s=(Star*)p1;
  Home *h=(Home*)p2;

  if (s->colour==h->colour)
    s->amihome=2;
}




/*--------------------------
       Pushing  */

TwoBody(d_Pushme_Star)
{
  if (!((Star*)p2)->amidead)
  {
    float ts=((Star*)p2)->rolling(cn.arg);
    MB.go(cn,0.4,0.8,ts);
  }
}

TwoBody(d_Pushme_Pushme)
{
  ((Pushme*)p1)->doubled=1;
  ((Pushme*)p2)->doubled=2;
  MB.go(cn,0.5,0.8,0);
} // I don't know how to do this.




/* ----------------------------------------------------------------
              Set Pieces blocking smaller objects */

TwoBody(d_Setpiece_Star)
{
  float ts;
  net_unit *b;
  Star *s2=(Star*)p2;

  if (!s2->amidead)
  {
    ts=s2->rolling(cn.arg);
    b=cn.nu;
    b->new_blake(0,0.8,ts);
    b->set_limit(1.5);
    b->components();
    if (b->norm_speed>10) s2->wake_up();
    MB.solve(b);
  }
}

TwoBody(d_Iceblock_Star)
{
  float ts;
  net_unit *b;
  Star *s2=(Star*)p2;

  if (!s2->amidead)
  {
    ts=s2->rolling(cn.arg);
    b=cn.nu;
    b->new_blake(0,0.05,ts);
    b->components();
    if (b->norm_speed>10) s2->wake_up();
    MB.solve(b);
  }
}

TwoBody(d_Setpiece_Block)
{
  MB.go(cn,0,0.8,0);
}

TwoBody(d_Platform_Star)
{
  net_unit *b;
  float ts;
  Platform *p=(Platform*)p1;
  Star *s=(Star*)p2;

  if (!s->amidead)
  {
    ts=s->rolling(cn.arg);
    if (cn.arg<-1 && cn.arg>-2.1)
    {
      b=cn.nu;
      b->new_blake(0,0.6,ts);
      b->set_limit(1.5);
      b->components();
      if (p->vy==0 && b->norm_speed>5) 
      {
        s->wake_up();
        p->impact=1;
      }
      MB.solve(b);
    }
  }
}

TwoBody(d_Platform_Block)
{
  net_unit *b;
  Platform *p=(Platform*)p1;

  if (cn.arg<-1 && cn.arg>-2.1)
  {
    b=cn.nu;
    b->new_blake(0,0.6,0);
    b->components();
    if (p->vy==0 && b->norm_speed>5) p->impact=1;
    
    MB.solve(b);
  }
}


/*  ---------------------------------------
     Set Pieces blocking rays of small particles */

TwoBody(d_Setpiece_Photon)
{
  p2->vx=p1->vx; p2->vy=p1->vy;
  p2->togo=2;
  cr + p2;
}

TwoBody(d_Mirror_Photon)
{
  net_unit *n;
  Mirror *m=(Mirror*)p1;
  Photon *f=(Photon*)p2;

  if (!m->broken && (argeq(m->way_thru,cn.arg) || m->way==5))
  {
    f->add_point(f->px,f->py);
    n=cn.nu;
    n->target_shear=0;
    if (m->way==5)
    {
      f->move(fl2i(2*n->cosarg),fl2i(2*n->sinarg)); // illegal, but if it works...
    }
    n->components();  
    n->J=2*n->norm_speed;
    n->p1p=0;
    n->impulse(0);
  }
  else
  {
    f->vx=0; f->vy=0;
    f->togo=2;
  }
  cr + f;
}

TwoBody(d_Setpiece_Particle)
{
  p2->togo=1;
  cr + p2;
}




/* -------------------------------------------------------
     End of 2-body routines */

void set_proc
(int i,int j,char checks,void (*proc)(Object*,Object*,c_notification,c_reporter&,char))
{ kinpair[i*max_xkinds+j].proc=proc;
  kinpair[i*max_xkinds+j].checks=checks;
}



void initialise_xkind_pairs()
{ 
  for(int j=0;j<max_xkinds*max_xkinds;j++)
    kinpair[j].checks=0;

  set_proc(xk_Object  ,xk_Object  ,0,d_Object_Object);     // null-shouldn't be called

  set_proc(xk_Star    ,xk_Magneton,4,d_Liftable_Magneton); // magnetism
  set_proc(xk_Pushme  ,xk_Magneton,4,d_Liftable_Magneton);
  set_proc(xk_Heavy   ,xk_Magneton,4,d_Liftable_Magneton);

  set_proc(xk_Chain   ,xk_Object , 1,d_Chain_Below);       // Chain constraint
  set_proc(xk_Chain   ,xk_Cutter  ,2,d_Chain_Cutter);      // chain cutting

  set_proc(xk_Mirror  ,xk_Bat     ,4,d_Mirror_Bat);        // bat smashes Mirror
  set_proc(xk_Door    ,xk_Quay    ,4,d_Door_Quay);         // door opening

  set_proc(xk_Gong    ,xk_Mallet  ,2,d_Gong_Mallet);       // mallet hits gong
  set_proc(xk_Star    ,xk_Phonon  ,4,d_Star_Phonon);       // star is awoken

  set_proc(xk_Star    ,xk_Spikes  ,4,d_Star_Spikes);       // death of star
  set_proc(xk_Star    ,xk_Zapper  ,6,d_Star_Zapper);
  set_proc(xk_Star    ,xk_Photon  ,6,d_Star_Photon);

  set_proc(xk_Laser   ,xk_Heavy   ,4,d_Laser_Heavy);       // destruction of Laser
  set_proc(xk_Laser   ,xk_Photon  ,4,d_Laser_Photon);

  set_proc(xk_Lever   ,xk_Platform,8,d_Lever_Platform);    // lever operation
  set_proc(xk_Star    ,xk_Teleport,2,d_Star_Teleport);     // teleport operation
  set_proc(xk_Star    ,xk_Ctrlbox ,2,d_Star_Ctrlbox);      // control box operation

  set_proc(xk_Star    ,xk_Gettable,2,d_Star_Gettable);     // collection
  set_proc(xk_Star    ,xk_Home    ,2,d_Star_Home);         // star at home

  set_proc(xk_Pushme  ,xk_Star    ,4,d_Pushme_Star);       // pushing
  set_proc(xk_Pushme  ,xk_Pushme  ,4,d_Pushme_Pushme);

  set_proc(xk_Setpiece,xk_Star    ,4,d_Setpiece_Star);     // blocking
  set_proc(xk_Iceblock,xk_Star    ,4,d_Iceblock_Star);  
  set_proc(xk_Magnet  ,xk_Star    ,4,d_Setpiece_Star);
  set_proc(xk_Laser   ,xk_Star    ,4,d_Setpiece_Star);
  set_proc(xk_Heavy   ,xk_Star    ,4,d_Setpiece_Star);
  set_proc(xk_Mirror  ,xk_Star    ,4,d_Setpiece_Star);
  set_proc(xk_Laser   ,xk_Pushme  ,4,d_Setpiece_Block);
  set_proc(xk_Magnet  ,xk_Pushme  ,4,d_Setpiece_Block);
  set_proc(xk_Setpiece,xk_Pushme  ,4,d_Setpiece_Block);
  set_proc(xk_Setpiece,xk_Heavy   ,4,d_Setpiece_Block);
  set_proc(xk_Setpiece,xk_Gettable,4,d_Setpiece_Block);
  set_proc(xk_Setpiece,xk_Magnet  ,4,d_Setpiece_Block);
  set_proc(xk_Platform,xk_Pushme  ,4,d_Platform_Block);
  set_proc(xk_Platform,xk_Heavy   ,4,d_Platform_Block);
  set_proc(xk_Platform,xk_Star    ,4,d_Platform_Star);

  set_proc(xk_Setpiece,xk_Magneton,4,d_Setpiece_Particle); // ray blocking
  set_proc(xk_Setpiece,xk_Phonon  ,4,d_Setpiece_Particle);
  set_proc(xk_Setpiece,xk_Photon  ,4,d_Setpiece_Photon);
  set_proc(xk_Pushme  ,xk_Photon  ,4,d_Setpiece_Photon);
  set_proc(xk_Heavy   ,xk_Photon  ,4,d_Setpiece_Photon);
  set_proc(xk_Mirror  ,xk_Photon  ,4,d_Mirror_Photon);
}

